"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List

from typing_extensions import TypedDict

from friendli.types import BaseModel


class DedicatedTokenizationSuccessTypedDict(TypedDict):
    tokens: List[int]
    r"""A list of token IDs."""


class DedicatedTokenizationSuccess(BaseModel):
    tokens: List[int]
    r"""A list of token IDs."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const

from .imagedata import ImageData, ImageDataTypedDict


class ImageContentTypedDict(TypedDict):
    image_url: ImageDataTypedDict
    type: Literal["image_url"]
    r"""The type of the message content."""


class ImageContent(BaseModel):
    image_url: ImageData

    TYPE: Annotated[
        Annotated[Literal["image_url"], AfterValidator(validate_const("image_url"))],
        pydantic.Field(alias="type"),
    ] = "image_url"
    r"""The type of the message content."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import weakref
from abc import abstractmethod
from typing import Any, Callable, Dict, Optional, Union, cast

import httpx

from friendli import models, utils
from friendli._hooks import SDKHooks
from friendli.container import AsyncContainer, SyncContainer
from friendli.dedicated import AsyncDedicated, SyncDedicated
from friendli.serverless import AsyncServerless, SyncServerless
from friendli.types import UNSET, OptionalNullable

from .basesdk import AsyncSDK, BaseSDK, SyncSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig


class BaseFriendli(BaseSDK):
    """Friendli Suite API Reference: This is an OpenAPI reference of Friendli Suite API."""

    def __init__(
        self,
        token: Optional[Union[Optional[str], Callable[[], Optional[str]]]] = None,
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        """Instantiates the SDK configuring it with the provided parameters.

        :param token: The token required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client()
            client_supplied = False
        assert issubclass(type(client), HttpClient), (
            "The provided client must implement the HttpClient protocol."
        )
        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient()
            async_client_supplied = False
        if debug_logger is None:
            debug_logger = get_default_logger()
        assert issubclass(type(async_client), AsyncHttpClient), (
            "The provided async_client must implement the AsyncHttpClient protocol."
        )
        security: Any = None
        if callable(token):
            security = lambda: models.Security(token=token())
        else:
            security = models.Security(token=token)
        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)
        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )
        hooks = SDKHooks()
        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url
        self.sdk_configuration.__dict__["_hooks"] = hooks
        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )
        self._init_sdks()

    @abstractmethod
    def _init_sdks(self):
        pass


class SyncFriendli(BaseFriendli, SyncSDK):
    serverless: SyncServerless
    dedicated: SyncDedicated
    container: SyncContainer

    def _init_sdks(self):
        self.serverless = SyncServerless(self.sdk_configuration)
        self.dedicated = SyncDedicated(self.sdk_configuration)
        self.container = SyncContainer(self.sdk_configuration)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.sdk_configuration.client is not None and (
            not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None


class AsyncFriendli(BaseFriendli, AsyncSDK):
    serverless: AsyncServerless
    dedicated: AsyncDedicated
    container: AsyncContainer

    def _init_sdks(self):
        self.serverless = AsyncServerless(self.sdk_configuration)
        self.dedicated = AsyncDedicated(self.sdk_configuration)
        self.container = AsyncContainer(self.sdk_configuration)

    async def __aenter__(self):
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.sdk_configuration.async_client is not None and (
            not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable
from friendli.utils import validate_const

from .streamedfunctionresult import (
    StreamedFunctionResult,
    StreamedFunctionResultTypedDict,
)


class StreamedToolCallResultTypedDict(TypedDict):
    function: StreamedFunctionResultTypedDict
    index: int
    r"""The index of tool call being generated."""
    type: Literal["function"]
    r"""The type of the tool."""
    id: NotRequired[Nullable[str]]
    r"""The ID of the tool call."""


class StreamedToolCallResult(BaseModel):
    function: StreamedFunctionResult

    index: int
    r"""The index of tool call being generated."""

    TYPE: Annotated[
        Annotated[Literal["function"], AfterValidator(validate_const("function"))],
        pydantic.Field(alias="type"),
    ] = "function"
    r"""The type of the tool."""

    id: OptionalNullable[str] = UNSET
    r"""The ID of the tool call."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id"]
        nullable_fields = ["id"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class FunctionResultTypedDict(TypedDict):
    arguments: str
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """
    name: str
    r"""The name of the function to call."""


class FunctionResult(BaseModel):
    arguments: str
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """

    name: str
    r"""The name of the function to call."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class ToolStatusParameterTypedDict(TypedDict):
    name: str
    r"""The name of the tool's function parameter."""
    value: str
    r"""The value of the tool's function parameter."""


class ToolStatusParameter(BaseModel):
    name: str
    r"""The name of the tool's function parameter."""

    value: str
    r"""The value of the tool's function parameter."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List, Literal

import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable
from friendli.utils import validate_const

from .chatchoice import ChatChoice, ChatChoiceTypedDict
from .chatusage import ChatUsage, ChatUsageTypedDict


class DedicatedChatCompleteSuccessTypedDict(TypedDict):
    choices: List[ChatChoiceTypedDict]
    created: int
    r"""The Unix timestamp (in seconds) for when the generation completed."""
    id: str
    r"""A unique ID of the chat completion."""
    usage: ChatUsageTypedDict
    object: Literal["chat.completion"]
    r"""The object type, which is always set to `chat.completion`."""
    model: NotRequired[Nullable[str]]
    r"""The model to generate the completion. For dedicated endpoints, it returns the endpoint id."""


class DedicatedChatCompleteSuccess(BaseModel):
    choices: List[ChatChoice]

    created: int
    r"""The Unix timestamp (in seconds) for when the generation completed."""

    id: str
    r"""A unique ID of the chat completion."""

    usage: ChatUsage

    OBJECT: Annotated[
        Annotated[
            Literal["chat.completion"],
            AfterValidator(validate_const("chat.completion")),
        ],
        pydantic.Field(alias="object"),
    ] = "chat.completion"
    r"""The object type, which is always set to `chat.completion`."""

    model: OptionalNullable[str] = UNSET
    r"""The model to generate the completion. For dedicated endpoints, it returns the endpoint id."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["model"]
        nullable_fields = ["model"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import logging
import os
from typing import Any, Protocol

import httpx


class Logger(Protocol):
    def debug(self, msg: str, *args: Any, **kwargs: Any) -> None:
        pass


class NoOpLogger:
    def debug(self, msg: str, *args: Any, **kwargs: Any) -> None:
        pass


def get_body_content(req: httpx.Request) -> str:
    return "<streaming body>" if not hasattr(req, "_content") else str(req.content)


def get_default_logger() -> Logger:
    if os.getenv("FRIENDLI_DEBUG"):
        logging.basicConfig(level=logging.DEBUG)
        return logging.getLogger("friendli")
    return NoOpLogger()



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Union

from typing_extensions import TypeAliasType

from .completionsstreamdedicatedmodelbodywithprompt import (
    CompletionsStreamDedicatedModelBodyWithPrompt,
    CompletionsStreamDedicatedModelBodyWithPromptTypedDict,
)
from .completionsstreamdedicatedmodelbodywithtokens import (
    CompletionsStreamDedicatedModelBodyWithTokens,
    CompletionsStreamDedicatedModelBodyWithTokensTypedDict,
)

DedicatedCompletionsStreamBodyTypedDict = TypeAliasType(
    "DedicatedCompletionsStreamBodyTypedDict",
    Union[
        CompletionsStreamDedicatedModelBodyWithPromptTypedDict,
        CompletionsStreamDedicatedModelBodyWithTokensTypedDict,
    ],
)


DedicatedCompletionsStreamBody = TypeAliasType(
    "DedicatedCompletionsStreamBody",
    Union[
        CompletionsStreamDedicatedModelBodyWithPrompt,
        CompletionsStreamDedicatedModelBodyWithTokens,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const


class ResponseFormatRegexTypedDict(TypedDict):
    schema_: str
    r"""The schema of the output. Lookaheads or lookbehinds (e.g., `\a`, `\z`, `^`, `$`, `(?=)`, `(?!)`, `(?<=...)`, `(?<!...)`) are not supported. Group specials (e.g., `\w`, `\W`, `\d`, `\D`, `\s`, `\S`) do not support non-ASCII characters. Unicode escape patterns (e.g., `\N`, `\p`, `\P`) are not supported. Additionally, conditional matching (`(?(`) and back-references can cause inefficiency.

    """
    type: Literal["regex"]
    r"""The type of the response format: `regex`"""


class ResponseFormatRegex(BaseModel):
    schema_: Annotated[str, pydantic.Field(alias="schema")]
    r"""The schema of the output. Lookaheads or lookbehinds (e.g., `\a`, `\z`, `^`, `$`, `(?=)`, `(?!)`, `(?<=...)`, `(?<!...)`) are not supported. Group specials (e.g., `\w`, `\W`, `\d`, `\D`, `\s`, `\S`) do not support non-ASCII characters. Unicode escape patterns (e.g., `\N`, `\p`, `\P`) are not supported. Additionally, conditional matching (`(?(`) and back-references can cause inefficiency.

    """

    TYPE: Annotated[
        Annotated[Literal["regex"], AfterValidator(validate_const("regex"))],
        pydantic.Field(alias="type"),
    ] = "regex"
    r"""The type of the response format: `regex`"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

import pydantic
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel


class KnowledgeRetrievedChunkTypedDict(TypedDict):
    r"""Knowledge retrieved chunk."""

    content_chunk_id: str
    r"""ID of specific text segment."""
    content_id: str
    r"""ID of content in knowledge-base."""
    score: float
    r"""Numerical relevance score of the retrieved result."""
    text: str
    r"""Retrieved text matching the search query."""


class KnowledgeRetrievedChunk(BaseModel):
    r"""Knowledge retrieved chunk."""

    content_chunk_id: Annotated[str, pydantic.Field(alias="contentChunkId")]
    r"""ID of specific text segment."""

    content_id: Annotated[str, pydantic.Field(alias="contentId")]
    r"""ID of content in knowledge-base."""

    score: float
    r"""Numerical relevance score of the retrieved result."""

    text: str
    r"""Retrieved text matching the search query."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List, Literal

from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable

from .toolstatuserror import ToolStatusError, ToolStatusErrorTypedDict
from .toolstatusfile import ToolStatusFile, ToolStatusFileTypedDict
from .toolstatusparameter import ToolStatusParameter, ToolStatusParameterTypedDict

Name = Literal[
    "math:calculator",
    "math:statistics",
    "math:calendar",
    "web:search",
    "web:url",
    "code:python-interpreter",
    "file:text",
]
r"""The name of the built-in tool."""

Status = Literal["STARTED", "UPDATING", "ENDED", "ERRORED"]
r"""Indicates the current execution status of the tool."""


class ToolStatusDataTypedDict(TypedDict):
    name: Name
    r"""The name of the built-in tool."""
    parameters: List[ToolStatusParameterTypedDict]
    status: Status
    r"""Indicates the current execution status of the tool."""
    timestamp: float
    r"""The Unix timestamp (in seconds) for when the event occurred."""
    tool_call_id: str
    r"""The ID of the tool call."""
    error: NotRequired[Nullable[ToolStatusErrorTypedDict]]
    files: NotRequired[Nullable[List[ToolStatusFileTypedDict]]]
    message: NotRequired[Nullable[str]]
    r"""Message generated by the tool's execution."""
    result: NotRequired[Nullable[str]]
    r"""The output from the tool's execution."""


class ToolStatusData(BaseModel):
    name: Name
    r"""The name of the built-in tool."""

    parameters: List[ToolStatusParameter]

    status: Status
    r"""Indicates the current execution status of the tool."""

    timestamp: float
    r"""The Unix timestamp (in seconds) for when the event occurred."""

    tool_call_id: str
    r"""The ID of the tool call."""

    error: OptionalNullable[ToolStatusError] = UNSET

    files: OptionalNullable[List[ToolStatusFile]] = UNSET

    message: OptionalNullable[str] = UNSET
    r"""Message generated by the tool's execution."""

    result: OptionalNullable[str] = UNSET
    r"""The output from the tool's execution."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["error", "files", "message", "result"]
        nullable_fields = ["error", "files", "message", "result"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Union

from typing_extensions import TypeAliasType

from .completionsstreambodywithprompt import (
    CompletionsStreamBodyWithPrompt,
    CompletionsStreamBodyWithPromptTypedDict,
)
from .completionsstreambodywithtokens import (
    CompletionsStreamBodyWithTokens,
    CompletionsStreamBodyWithTokensTypedDict,
)

ContainerCompletionsStreamBodyTypedDict = TypeAliasType(
    "ContainerCompletionsStreamBodyTypedDict",
    Union[
        CompletionsStreamBodyWithPromptTypedDict,
        CompletionsStreamBodyWithTokensTypedDict,
    ],
)


ContainerCompletionsStreamBody = TypeAliasType(
    "ContainerCompletionsStreamBody",
    Union[CompletionsStreamBodyWithPrompt, CompletionsStreamBodyWithTokens],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class AssistantMessageToolCallFunctionTypedDict(TypedDict):
    arguments: str
    r"""The arguments of function in JSON schema format to call the function."""
    name: str
    r"""The name of function"""


class AssistantMessageToolCallFunction(BaseModel):
    arguments: str
    r"""The arguments of function in JSON schema format to call the function."""

    name: str
    r"""The name of function"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class ToolStatusErrorTypedDict(TypedDict):
    msg: str
    r"""The message of error."""
    type: str
    r"""The type of error encountered during the tool's execution."""


class ToolStatusError(BaseModel):
    msg: str
    r"""The message of error."""

    type: str
    r"""The type of error encountered during the tool's execution."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Union

from pydantic import Discriminator, Tag
from typing_extensions import Annotated, TypeAliasType

from friendli.utils import get_discriminator

from .audiocontent import AudioContent, AudioContentTypedDict
from .imagecontent import ImageContent, ImageContentTypedDict
from .textcontent import TextContent, TextContentTypedDict
from .videocontent import VideoContent, VideoContentTypedDict

UserMessageContentMultiModalTypedDict = TypeAliasType(
    "UserMessageContentMultiModalTypedDict",
    Union[
        TextContentTypedDict,
        AudioContentTypedDict,
        ImageContentTypedDict,
        VideoContentTypedDict,
    ],
)


UserMessageContentMultiModal = Annotated[
    Union[
        Annotated[AudioContent, Tag("audio_url")],
        Annotated[ImageContent, Tag("image_url")],
        Annotated[TextContent, Tag("text")],
        Annotated[VideoContent, Tag("video_url")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable

DedicatedImageGenerationBodyResponseFormat = Literal["url", "raw", "png", "jpeg", "jpg"]
r"""The format in which the generated image(s) will be returned. One of `url(default)`, `raw`, `png`, `jpeg`, and `jpg`."""


class DedicatedImageGenerationBodyTypedDict(TypedDict):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, use the format \"YOUR_ENDPOINT_ID:YOUR_ADAPTER_ROUTE\". Otherwise, you can just use \"YOUR_ENDPOINT_ID\" alone."""
    prompt: str
    r"""A text description of the desired image(s)."""
    num_inference_steps: NotRequired[Nullable[int]]
    r"""The number of inference steps to use during image generation. Supported range: [1, 10]. Defaults to 4."""
    response_format: NotRequired[Nullable[DedicatedImageGenerationBodyResponseFormat]]
    r"""The format in which the generated image(s) will be returned. One of `url(default)`, `raw`, `png`, `jpeg`, and `jpg`."""
    seed: NotRequired[Nullable[int]]
    r"""The seed to use for image generation."""


class DedicatedImageGenerationBody(BaseModel):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, use the format \"YOUR_ENDPOINT_ID:YOUR_ADAPTER_ROUTE\". Otherwise, you can just use \"YOUR_ENDPOINT_ID\" alone."""

    prompt: str
    r"""A text description of the desired image(s)."""

    num_inference_steps: OptionalNullable[int] = 4
    r"""The number of inference steps to use during image generation. Supported range: [1, 10]. Defaults to 4."""

    response_format: OptionalNullable[DedicatedImageGenerationBodyResponseFormat] = (
        "url"
    )
    r"""The format in which the generated image(s) will be returned. One of `url(default)`, `raw`, `png`, `jpeg`, and `jpg`."""

    seed: OptionalNullable[int] = UNSET
    r"""The seed to use for image generation."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["num_inference_steps", "response_format", "seed"]
        nullable_fields = ["num_inference_steps", "response_format", "seed"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable

from .acceleratorrequirement import (
    AcceleratorRequirement,
    AcceleratorRequirementTypedDict,
)
from .autoscalingpolicy import AutoscalingPolicy, AutoscalingPolicyTypedDict


class DedicatedEndpointWandbArtifactCreateBodyTypedDict(TypedDict):
    r"""Endpoint create request."""

    wandb_artifact_version_name: str
    r"""The specific model artifact version from Weights & Biases. The referred artifact will be used to create a new endpoint in Friendli Dedicated Endpoints or rollout an existing one."""
    accelerator: NotRequired[Nullable[AcceleratorRequirementTypedDict]]
    r"""Specifies the instance type for the endpoint."""
    autoscaling_policy: NotRequired[Nullable[AutoscalingPolicyTypedDict]]
    r"""Defines autoscaling settings for the endpoint."""
    idempotency_key: NotRequired[Nullable[str]]
    r"""Used by Friendli Dedicated Endpoints to track which webhook automation triggered an endpoint rollout. If the `idempotencyKey` is provided, the API will check if the endpoint already exists, and rollout the existing endpoint if it does. In such cases, the `projectId` must be provided. Any unique value can be used."""
    name: NotRequired[Nullable[str]]
    r"""Specifies the name of your endpoint. If not provided, a name will be automatically generated for you."""
    project_id: NotRequired[Nullable[str]]
    r"""Specifies where endpoint will be created in your Friendli Dedicated Endpoints. If not provided, a new project will be created within your default team."""


class DedicatedEndpointWandbArtifactCreateBody(BaseModel):
    r"""Endpoint create request."""

    wandb_artifact_version_name: Annotated[
        str, pydantic.Field(alias="wandbArtifactVersionName")
    ]
    r"""The specific model artifact version from Weights & Biases. The referred artifact will be used to create a new endpoint in Friendli Dedicated Endpoints or rollout an existing one."""

    accelerator: OptionalNullable[AcceleratorRequirement] = UNSET
    r"""Specifies the instance type for the endpoint."""

    autoscaling_policy: Annotated[
        OptionalNullable[AutoscalingPolicy], pydantic.Field(alias="autoscalingPolicy")
    ] = UNSET
    r"""Defines autoscaling settings for the endpoint."""

    idempotency_key: Annotated[
        OptionalNullable[str], pydantic.Field(alias="idempotencyKey")
    ] = UNSET
    r"""Used by Friendli Dedicated Endpoints to track which webhook automation triggered an endpoint rollout. If the `idempotencyKey` is provided, the API will check if the endpoint already exists, and rollout the existing endpoint if it does. In such cases, the `projectId` must be provided. Any unique value can be used."""

    name: OptionalNullable[str] = UNSET
    r"""Specifies the name of your endpoint. If not provided, a name will be automatically generated for you."""

    project_id: Annotated[OptionalNullable[str], pydantic.Field(alias="projectId")] = (
        UNSET
    )
    r"""Specifies where endpoint will be created in your Friendli Dedicated Endpoints. If not provided, a new project will be created within your default team."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "accelerator",
            "autoscalingPolicy",
            "idempotencyKey",
            "name",
            "projectId",
        ]
        nullable_fields = [
            "accelerator",
            "autoscalingPolicy",
            "idempotencyKey",
            "name",
            "projectId",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel

from .streamedchatdata import StreamedChatData, StreamedChatDataTypedDict


class ContainerChatCompletionStreamSuccessTypedDict(TypedDict):
    data: StreamedChatDataTypedDict


class ContainerChatCompletionStreamSuccess(BaseModel):
    data: StreamedChatData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class ServerlessDetokenizationSuccessTypedDict(TypedDict):
    text: str
    r"""Detokenized text output."""


class ServerlessDetokenizationSuccess(BaseModel):
    text: str
    r"""Detokenized text output."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List, Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const


class FileBuiltInToolTypedDict(TypedDict):
    files: List[str]
    r"""A List of file IDs. For now, only one file is supported."""
    type: Literal["file:text"]
    r"""The type of the file parser tool. Only .txt and .pdf files are supported."""


class FileBuiltInTool(BaseModel):
    files: List[str]
    r"""A List of file IDs. For now, only one file is supported."""

    TYPE: Annotated[
        Annotated[Literal["file:text"], AfterValidator(validate_const("file:text"))],
        pydantic.Field(alias="type"),
    ] = "file:text"
    r"""The type of the file parser tool. Only .txt and .pdf files are supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from abc import abstractmethod
from typing import Mapping, Optional, Union

from friendli import models, utils
from friendli._hooks import HookContext
from friendli.types import UNSET, OptionalNullable
from friendli.utils import get_security_from_env

from .basesdk import AsyncSDK, BaseSDK, SyncSDK


class BaseEndpoint(BaseSDK):
    pass


class SyncEndpoint(BaseEndpoint, SyncSDK):
    def wandb_artifact_create(
        self,
        *,
        wandb_artifact_version_name: str,
        x_friendli_team: OptionalNullable[str] = UNSET,
        accelerator: OptionalNullable[
            Union[models.AcceleratorRequirement, models.AcceleratorRequirementTypedDict]
        ] = UNSET,
        autoscaling_policy: OptionalNullable[
            Union[models.AutoscalingPolicy, models.AutoscalingPolicyTypedDict]
        ] = UNSET,
        idempotency_key: OptionalNullable[str] = UNSET,
        name: OptionalNullable[str] = UNSET,
        project_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DedicatedEndpointWandbArtifactCreateResponse:
        """Create endpoint from W&B artifact

        Create an endpoint from Weights & Biases artifact. If the idempotency key is provided, the API will check if the endpoint already exists, and rollout the existing endpoint if it does. In such cases, the project id must be provided.

        :param wandb_artifact_version_name: The specific model artifact version from Weights & Biases. The referred artifact will be used to create a new endpoint in Friendli Dedicated Endpoints or rollout an existing one.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param accelerator: Specifies the instance type for the endpoint.
        :param autoscaling_policy: Defines autoscaling settings for the endpoint.
        :param idempotency_key: Used by Friendli Dedicated Endpoints to track which webhook automation triggered an endpoint rollout. If the `idempotencyKey` is provided, the API will check if the endpoint already exists, and rollout the existing endpoint if it does. In such cases, the `projectId` must be provided. Any unique value can be used.
        :param name: Specifies the name of your endpoint. If not provided, a name will be automatically generated for you.
        :param project_id: Specifies where endpoint will be created in your Friendli Dedicated Endpoints. If not provided, a new project will be created within your default team.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms
        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        request = models.DedicatedEndpointWandbArtifactCreateRequest(
            x_friendli_team=x_friendli_team,
            dedicated_endpoint_wandb_artifact_create_body=models.DedicatedEndpointWandbArtifactCreateBody(
                wandb_artifact_version_name=wandb_artifact_version_name,
                accelerator=utils.get_pydantic_model(
                    accelerator, OptionalNullable[models.AcceleratorRequirement]
                ),
                autoscaling_policy=utils.get_pydantic_model(
                    autoscaling_policy, OptionalNullable[models.AutoscalingPolicy]
                ),
                idempotency_key=idempotency_key,
                name=name,
                project_id=project_id,
            ),
        )
        req = self._build_request(
            method="POST",
            path="/dedicated/v1/endpoint/wandb-artifact-create",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_endpoint_wandb_artifact_create_body,
                False,
                False,
                "json",
                models.DedicatedEndpointWandbArtifactCreateBody,
            ),
            timeout_ms=timeout_ms,
        )
        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )
        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])
        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="dedicatedEndpointWandbArtifactCreate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.DedicatedEndpointWandbArtifactCreateResponse
            )
        if utils.match_response(http_res, ["400", "422", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )


class AsyncEndpoint(BaseEndpoint, AsyncSDK):
    async def wandb_artifact_create(
        self,
        *,
        wandb_artifact_version_name: str,
        x_friendli_team: OptionalNullable[str] = UNSET,
        accelerator: OptionalNullable[
            Union[models.AcceleratorRequirement, models.AcceleratorRequirementTypedDict]
        ] = UNSET,
        autoscaling_policy: OptionalNullable[
            Union[models.AutoscalingPolicy, models.AutoscalingPolicyTypedDict]
        ] = UNSET,
        idempotency_key: OptionalNullable[str] = UNSET,
        name: OptionalNullable[str] = UNSET,
        project_id: OptionalNullable[str] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DedicatedEndpointWandbArtifactCreateResponse:
        """Create endpoint from W&B artifact

        Create an endpoint from Weights & Biases artifact. If the idempotency key is provided, the API will check if the endpoint already exists, and rollout the existing endpoint if it does. In such cases, the project id must be provided.

        :param wandb_artifact_version_name: The specific model artifact version from Weights & Biases. The referred artifact will be used to create a new endpoint in Friendli Dedicated Endpoints or rollout an existing one.
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param accelerator: Specifies the instance type for the endpoint.
        :param autoscaling_policy: Defines autoscaling settings for the endpoint.
        :param idempotency_key: Used by Friendli Dedicated Endpoints to track which webhook automation triggered an endpoint rollout. If the `idempotencyKey` is provided, the API will check if the endpoint already exists, and rollout the existing endpoint if it does. In such cases, the `projectId` must be provided. Any unique value can be used.
        :param name: Specifies the name of your endpoint. If not provided, a name will be automatically generated for you.
        :param project_id: Specifies where endpoint will be created in your Friendli Dedicated Endpoints. If not provided, a new project will be created within your default team.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms
        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        request = models.DedicatedEndpointWandbArtifactCreateRequest(
            x_friendli_team=x_friendli_team,
            dedicated_endpoint_wandb_artifact_create_body=models.DedicatedEndpointWandbArtifactCreateBody(
                wandb_artifact_version_name=wandb_artifact_version_name,
                accelerator=utils.get_pydantic_model(
                    accelerator, OptionalNullable[models.AcceleratorRequirement]
                ),
                autoscaling_policy=utils.get_pydantic_model(
                    autoscaling_policy, OptionalNullable[models.AutoscalingPolicy]
                ),
                idempotency_key=idempotency_key,
                name=name,
                project_id=project_id,
            ),
        )
        req = self._build_request(
            method="POST",
            path="/dedicated/v1/endpoint/wandb-artifact-create",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_endpoint_wandb_artifact_create_body,
                False,
                False,
                "json",
                models.DedicatedEndpointWandbArtifactCreateBody,
            ),
            timeout_ms=timeout_ms,
        )
        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )
        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])
        http_res = await self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="dedicatedEndpointWandbArtifactCreate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.DedicatedEndpointWandbArtifactCreateResponse
            )
        if utils.match_response(http_res, ["400", "422", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel

from .streamedcompletiondata import (
    StreamedCompletionData,
    StreamedCompletionDataTypedDict,
)


class ServerlessCompletionsStreamSuccessTypedDict(TypedDict):
    data: StreamedCompletionDataTypedDict


class ServerlessCompletionsStreamSuccess(BaseModel):
    data: StreamedCompletionData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from pydantic import model_serializer
from typing_extensions import TypedDict

from friendli.types import UNSET_SENTINEL, BaseModel, Nullable

from .functionality import Functionality, FunctionalityTypedDict
from .pricingmodel import PricingModel, PricingModelTypedDict


class ModelCatalogResponseItemTypedDict(TypedDict):
    r"""Model listing in `/models` API."""

    context_length: int
    created: int
    description: str
    functionality: FunctionalityTypedDict
    r"""Functionality of the model."""
    hugging_face_url: str
    id: str
    license: str
    name: str
    policy: Nullable[str]
    pricing: PricingModelTypedDict
    r"""Price per 1M tokens in USD currency."""


class ModelCatalogResponseItem(BaseModel):
    r"""Model listing in `/models` API."""

    context_length: int

    created: int

    description: str

    functionality: Functionality
    r"""Functionality of the model."""

    hugging_face_url: str

    id: str

    license: str

    name: str

    policy: Nullable[str]

    pricing: PricingModel
    r"""Price per 1M tokens in USD currency."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["policy"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable

from .completionslogprobs import CompletionsLogprobs, CompletionsLogprobsTypedDict

StreamedCompletionsChoiceFinishReason = Literal["stop", "length", "null"]
r"""Termination condition of the generation. `stop` means the API returned the full completions generated by the model without running into any limits. `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length."""


class StreamedCompletionsChoiceTypedDict(TypedDict):
    index: int
    r"""The index of the choice in the list of generated choices."""
    text: str
    r"""Generated text output."""
    finish_reason: NotRequired[Nullable[StreamedCompletionsChoiceFinishReason]]
    r"""Termination condition of the generation. `stop` means the API returned the full completions generated by the model without running into any limits. `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length."""
    logprobs: NotRequired[Nullable[CompletionsLogprobsTypedDict]]
    token: NotRequired[Nullable[int]]
    r"""The token."""


class StreamedCompletionsChoice(BaseModel):
    index: int
    r"""The index of the choice in the list of generated choices."""

    text: str
    r"""Generated text output."""

    finish_reason: OptionalNullable[StreamedCompletionsChoiceFinishReason] = UNSET
    r"""Termination condition of the generation. `stop` means the API returned the full completions generated by the model without running into any limits. `length` means the generation exceeded `max_tokens` or the conversation exceeded the max context length."""

    logprobs: OptionalNullable[CompletionsLogprobs] = UNSET

    token: OptionalNullable[int] = UNSET
    r"""The token."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["finish_reason", "logprobs", "token"]
        nullable_fields = ["finish_reason", "logprobs", "token"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class PricingModelTypedDict(TypedDict):
    r"""Price per 1M tokens in USD currency."""

    input: float
    r"""Price per input token"""
    output: float
    r"""Price per output token"""


class PricingModel(BaseModel):
    r"""Price per 1M tokens in USD currency."""

    input: float
    r"""Price per input token"""

    output: float
    r"""Price per output token"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class ServerlessTokenizationBodyTypedDict(TypedDict):
    model: str
    r"""Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models)."""
    prompt: str
    r"""Input text prompt to tokenize."""


class ServerlessTokenizationBody(BaseModel):
    model: str
    r"""Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models)."""

    prompt: str
    r"""Input text prompt to tokenize."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Union

from typing_extensions import TypeAliasType

from .completionsstreamserverlessmodelbodywithprompt import (
    CompletionsStreamServerlessModelBodyWithPrompt,
    CompletionsStreamServerlessModelBodyWithPromptTypedDict,
)
from .completionsstreamserverlessmodelbodywithtokens import (
    CompletionsStreamServerlessModelBodyWithTokens,
    CompletionsStreamServerlessModelBodyWithTokensTypedDict,
)

ServerlessCompletionsStreamBodyTypedDict = TypeAliasType(
    "ServerlessCompletionsStreamBodyTypedDict",
    Union[
        CompletionsStreamServerlessModelBodyWithPromptTypedDict,
        CompletionsStreamServerlessModelBodyWithTokensTypedDict,
    ],
)


ServerlessCompletionsStreamBody = TypeAliasType(
    "ServerlessCompletionsStreamBody",
    Union[
        CompletionsStreamServerlessModelBodyWithPrompt,
        CompletionsStreamServerlessModelBodyWithTokens,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class VideoDataTypedDict(TypedDict):
    url: str
    r"""The URL of the video."""


class VideoData(BaseModel):
    url: str
    r"""The URL of the video."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable
from friendli.utils import FieldMetadata, HeaderMetadata, RequestMetadata

from .dedicatedimagegenerationbody import (
    DedicatedImageGenerationBody,
    DedicatedImageGenerationBodyTypedDict,
)


class DedicatedImagesGenerateRequestTypedDict(TypedDict):
    dedicated_image_generation_body: DedicatedImageGenerationBodyTypedDict
    x_friendli_team: NotRequired[Nullable[str]]
    r"""ID of team to run requests as (optional parameter)."""


class DedicatedImagesGenerateRequest(BaseModel):
    dedicated_image_generation_body: Annotated[
        DedicatedImageGenerationBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_friendli_team: Annotated[
        OptionalNullable[str],
        pydantic.Field(alias="X-Friendli-Team"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = UNSET
    r"""ID of team to run requests as (optional parameter)."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["X-Friendli-Team"]
        nullable_fields = ["X-Friendli-Team"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Union

from pydantic import Discriminator, Tag
from typing_extensions import Annotated, TypeAliasType

from friendli.utils import get_discriminator

from .filebuiltintool import FileBuiltInTool, FileBuiltInToolTypedDict
from .otherbuiltintool import OtherBuiltInTool, OtherBuiltInToolTypedDict
from .tool import Tool, ToolTypedDict

ToolAssistedChatToolTypedDict = TypeAliasType(
    "ToolAssistedChatToolTypedDict",
    Union[OtherBuiltInToolTypedDict, ToolTypedDict, FileBuiltInToolTypedDict],
)


ToolAssistedChatTool = Annotated[
    Union[
        Annotated[OtherBuiltInTool, Tag("code:python-interpreter")],
        Annotated[FileBuiltInTool, Tag("file:text")],
        Annotated[Tool, Tag("function")],
        Annotated[OtherBuiltInTool, Tag("math:calculator")],
        Annotated[OtherBuiltInTool, Tag("math:calendar")],
        Annotated[OtherBuiltInTool, Tag("math:statistics")],
        Annotated[OtherBuiltInTool, Tag("web:search")],
        Annotated[OtherBuiltInTool, Tag("web:url")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Union

from typing_extensions import TypeAliasType

from .completionsserverlessmodelbodywithprompt import (
    CompletionsServerlessModelBodyWithPrompt,
    CompletionsServerlessModelBodyWithPromptTypedDict,
)
from .completionsserverlessmodelbodywithtokens import (
    CompletionsServerlessModelBodyWithTokens,
    CompletionsServerlessModelBodyWithTokensTypedDict,
)

ServerlessCompletionsBodyTypedDict = TypeAliasType(
    "ServerlessCompletionsBodyTypedDict",
    Union[
        CompletionsServerlessModelBodyWithPromptTypedDict,
        CompletionsServerlessModelBodyWithTokensTypedDict,
    ],
)


ServerlessCompletionsBody = TypeAliasType(
    "ServerlessCompletionsBody",
    Union[
        CompletionsServerlessModelBodyWithPrompt,
        CompletionsServerlessModelBodyWithTokens,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List

from typing_extensions import TypedDict

from friendli.types import BaseModel


class TopLogprobsTypedDict(TypedDict):
    pass


class TopLogprobs(BaseModel):
    pass


class CompletionsLogprobsTypedDict(TypedDict):
    text_offset: List[int]
    r"""The starting character position of each token in the generated text, useful for mapping tokens back to their exact location for detailed analysis."""
    token_logprobs: List[float]
    r"""The log probabilities of each generated token, indicating the model's confidence in selecting each token."""
    tokens: List[str]
    r"""A list of individual tokens generated in the completion, representing segments of text such as words or pieces of words."""
    top_logprobs: List[TopLogprobsTypedDict]
    r"""A list of dictionaries, where each dictionary represents the top alternative tokens considered by the model at a specific position in the generated text, along with their log probabilities. The number of items in each dictionary matches the value of `logprobs`."""


class CompletionsLogprobs(BaseModel):
    text_offset: List[int]
    r"""The starting character position of each token in the generated text, useful for mapping tokens back to their exact location for detailed analysis."""

    token_logprobs: List[float]
    r"""The log probabilities of each generated token, indicating the model's confidence in selecting each token."""

    tokens: List[str]
    r"""A list of individual tokens generated in the completion, representing segments of text such as words or pieces of words."""

    top_logprobs: List[TopLogprobs]
    r"""A list of dictionaries, where each dictionary represents the top alternative tokens considered by the model at a specific position in the generated text, along with their log probabilities. The number of items in each dictionary matches the value of `logprobs`."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Union

from pydantic import Discriminator, Tag
from typing_extensions import Annotated, TypeAliasType

from friendli.utils import get_discriminator

from .responseformatjsonobject import (
    ResponseFormatJSONObject,
    ResponseFormatJSONObjectTypedDict,
)
from .responseformatjsonschema import (
    ResponseFormatJSONSchema,
    ResponseFormatJSONSchemaTypedDict,
)
from .responseformatregex import ResponseFormatRegex, ResponseFormatRegexTypedDict
from .responseformattext import ResponseFormatText, ResponseFormatTextTypedDict

ResponseFormatTypedDict = TypeAliasType(
    "ResponseFormatTypedDict",
    Union[
        ResponseFormatJSONObjectTypedDict,
        ResponseFormatTextTypedDict,
        ResponseFormatJSONSchemaTypedDict,
        ResponseFormatRegexTypedDict,
    ],
)
r"""The enforced format of the model's output.

Note that the content of the output message may be truncated if it exceeds the `max_tokens`. You can check this by verifying that the `finish_reason` of the output message is `length`.

For more detailed information, please refer [here](https://friendli.ai/docs/guides/serverless_endpoints/structured-outputs).

***Important***
You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`).
Otherwise, the model may result in an unending stream of whitespace or other characters.

**When `response_format` is specified, `min_tokens` field is unsupported.**


"""


ResponseFormat = Annotated[
    Union[
        Annotated[ResponseFormatJSONObject, Tag("json_object")],
        Annotated[ResponseFormatJSONSchema, Tag("json_schema")],
        Annotated[ResponseFormatRegex, Tag("regex")],
        Annotated[ResponseFormatText, Tag("text")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]
r"""The enforced format of the model's output.

Note that the content of the output message may be truncated if it exceeds the `max_tokens`. You can check this by verifying that the `finish_reason` of the output message is `length`.

For more detailed information, please refer [here](https://friendli.ai/docs/guides/serverless_endpoints/structured-outputs).

***Important***
You must explicitly instruct the model to produce the desired output format using a system prompt or user message (e.g., `You are an API generating a valid JSON as output.`).
Otherwise, the model may result in an unending stream of whitespace or other characters.

**When `response_format` is specified, `min_tokens` field is unsupported.**


"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from abc import abstractmethod

from friendli.friendli_container_chat import (
    AsyncFriendliContainerChat,
    SyncFriendliContainerChat,
)
from friendli.friendli_container_completions import (
    AsyncFriendliContainerCompletions,
    SyncFriendliContainerCompletions,
)
from friendli.friendli_container_token import (
    AsyncFriendliContainerToken,
    SyncFriendliContainerToken,
)

from .basesdk import AsyncSDK, BaseSDK, SyncSDK
from .sdkconfiguration import SDKConfiguration


class BaseContainer(BaseSDK):
    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    @abstractmethod
    def _init_sdks(self):
        pass


class SyncContainer(BaseContainer, SyncSDK):
    chat: SyncFriendliContainerChat
    completions: SyncFriendliContainerCompletions
    token: SyncFriendliContainerToken

    def _init_sdks(self):
        self.chat = SyncFriendliContainerChat(self.sdk_configuration)
        self.completions = SyncFriendliContainerCompletions(self.sdk_configuration)
        self.token = SyncFriendliContainerToken(self.sdk_configuration)


class AsyncContainer(BaseContainer, AsyncSDK):
    chat: AsyncFriendliContainerChat
    completions: AsyncFriendliContainerCompletions
    token: AsyncFriendliContainerToken

    def _init_sdks(self):
        self.chat = AsyncFriendliContainerChat(self.sdk_configuration)
        self.completions = AsyncFriendliContainerCompletions(self.sdk_configuration)
        self.token = AsyncFriendliContainerToken(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Optional

from typing_extensions import NotRequired, TypedDict

from friendli.types import BaseModel


class FunctionalityTypedDict(TypedDict):
    r"""Functionality of the model."""

    parallel_tool_call: bool
    tool_call: bool
    structured_output: NotRequired[bool]


class Functionality(BaseModel):
    r"""Functionality of the model."""

    parallel_tool_call: bool

    tool_call: bool

    structured_output: Optional[bool] = True



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const

from .toolstatusdata import ToolStatusData, ToolStatusDataTypedDict


class StreamedToolAssistedChatToolStatusTypedDict(TypedDict):
    data: ToolStatusDataTypedDict
    event: Literal["tool_status"]


class StreamedToolAssistedChatToolStatus(BaseModel):
    data: ToolStatusData

    EVENT: Annotated[
        Annotated[
            Literal["tool_status"], AfterValidator(validate_const("tool_status"))
        ],
        pydantic.Field(alias="event"),
    ] = "tool_status"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

from typing_extensions import TypedDict

from friendli.types import BaseModel

B64ImageItemResponseFormat = Literal["url", "raw", "png", "jpeg", "jpg"]
r"""The format of the generated image. One of `url(default)`, `raw`, `png`, `jpeg`, and `jpg`."""


class B64ImageItemTypedDict(TypedDict):
    b64_json: bytes
    r"""The base64-encoded JSON of the generated image."""
    response_format: B64ImageItemResponseFormat
    r"""The format of the generated image. One of `url(default)`, `raw`, `png`, `jpeg`, and `jpg`."""
    seed: int
    r"""The seed used during image generation."""


class B64ImageItem(BaseModel):
    b64_json: bytes
    r"""The base64-encoded JSON of the generated image."""

    response_format: B64ImageItemResponseFormat
    r"""The format of the generated image. One of `url(default)`, `raw`, `png`, `jpeg`, and `jpg`."""

    seed: int
    r"""The seed used during image generation."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

from typing_extensions import TypedDict

from friendli.types import BaseModel

Type = Literal[
    "math:calculator",
    "math:statistics",
    "math:calendar",
    "web:search",
    "web:url",
    "code:python-interpreter",
]
r"""The type of the built-in tool."""


class OtherBuiltInToolTypedDict(TypedDict):
    type: Type
    r"""The type of the built-in tool."""


class OtherBuiltInTool(BaseModel):
    type: Type
    r"""The type of the built-in tool."""



from types import TracebackType
from typing import Any, NoReturn, Optional, Type, TypeVar, Union

import httpx

from .httpclient import AsyncHttpClient, HttpClient

T = TypeVar("T", bound="ErrorThrowingSyncClient")
U = TypeVar("U", bound="ErrorThrowingAsyncClient")


class MethodUsedInWrongPlaceError(Exception):
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class ErrorThrowingSyncClient(HttpClient):
    def send(
        self,
        request: httpx.Request,
        *,
        stream: bool = False,
        auth: Union[
            httpx._types.AuthTypes, httpx._client.UseClientDefault, None
        ] = httpx.USE_CLIENT_DEFAULT,
        follow_redirects: Union[
            bool, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
    ) -> httpx.Response:
        self._raise_error()

    def build_request(
        self,
        method: str,
        url: httpx._types.URLTypes,
        *,
        content: Optional[httpx._types.RequestContent] = None,
        data: Optional[httpx._types.RequestData] = None,
        files: Optional[httpx._types.RequestFiles] = None,
        json: Optional[Any] = None,
        params: Optional[httpx._types.QueryParamTypes] = None,
        headers: Optional[httpx._types.HeaderTypes] = None,
        cookies: Optional[httpx._types.CookieTypes] = None,
        timeout: Union[
            httpx._types.TimeoutTypes, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
        extensions: Optional[httpx._types.RequestExtensions] = None,
    ) -> httpx.Request:
        self._raise_error()

    def __enter__(self: T) -> T:
        return self

    def __exit__(
        self,
        exc_type: Union[Type[BaseException], None],
        exc_val: Union[BaseException, None],
        exc_tb: Union[TracebackType, None],
    ) -> None:
        pass

    def _raise_error(self) -> NoReturn:
        raise MethodUsedInWrongPlaceError(
            "Sync operations are not supported in AsyncFriendli."
        )


class ErrorThrowingAsyncClient(AsyncHttpClient):
    async def send(
        self,
        request: httpx.Request,
        *,
        stream: bool = False,
        auth: Union[
            httpx._types.AuthTypes, httpx._client.UseClientDefault, None
        ] = httpx.USE_CLIENT_DEFAULT,
        follow_redirects: Union[
            bool, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
    ) -> httpx.Response:
        self._raise_error()

    def build_request(
        self,
        method: str,
        url: httpx._types.URLTypes,
        *,
        content: Optional[httpx._types.RequestContent] = None,
        data: Optional[httpx._types.RequestData] = None,
        files: Optional[httpx._types.RequestFiles] = None,
        json: Optional[Any] = None,
        params: Optional[httpx._types.QueryParamTypes] = None,
        headers: Optional[httpx._types.HeaderTypes] = None,
        cookies: Optional[httpx._types.CookieTypes] = None,
        timeout: Union[
            httpx._types.TimeoutTypes, httpx._client.UseClientDefault
        ] = httpx.USE_CLIENT_DEFAULT,
        extensions: Optional[httpx._types.RequestExtensions] = None,
    ) -> httpx.Request:
        self._raise_error()

    async def __aenter__(self: U) -> U:
        return self

    async def __aexit__(
        self,
        exc_type: Union[Type[BaseException], None],
        exc_val: Union[BaseException, None],
        exc_tb: Union[TracebackType, None],
    ) -> None:
        pass

    def _raise_error(self) -> NoReturn:
        raise MethodUsedInWrongPlaceError(
            "Async operations are not supported in SyncFriendli."
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from abc import abstractmethod

from friendli.chat import AsyncChat, SyncChat
from friendli.completions import AsyncCompletions, SyncCompletions
from friendli.knowledge import AsyncKnowledge, SyncKnowledge
from friendli.model import AsyncModel, SyncModel
from friendli.token import AsyncToken, SyncToken
from friendli.toolassistedchat import AsyncToolAssistedChat, SyncToolAssistedChat

from .basesdk import AsyncSDK, BaseSDK, SyncSDK
from .sdkconfiguration import SDKConfiguration


class BaseServerless(BaseSDK):
    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    @abstractmethod
    def _init_sdks(self):
        pass


class SyncServerless(BaseServerless, SyncSDK):
    chat: SyncChat
    completions: SyncCompletions
    token: SyncToken
    knowledge: SyncKnowledge
    model: SyncModel
    tool_assisted_chat: SyncToolAssistedChat

    def _init_sdks(self):
        self.chat = SyncChat(self.sdk_configuration)
        self.completions = SyncCompletions(self.sdk_configuration)
        self.token = SyncToken(self.sdk_configuration)
        self.knowledge = SyncKnowledge(self.sdk_configuration)
        self.model = SyncModel(self.sdk_configuration)
        self.tool_assisted_chat = SyncToolAssistedChat(self.sdk_configuration)


class AsyncServerless(BaseServerless, AsyncSDK):
    chat: AsyncChat
    completions: AsyncCompletions
    token: AsyncToken
    knowledge: AsyncKnowledge
    model: AsyncModel
    tool_assisted_chat: AsyncToolAssistedChat

    def _init_sdks(self):
        self.chat = AsyncChat(self.sdk_configuration)
        self.completions = AsyncCompletions(self.sdk_configuration)
        self.token = AsyncToken(self.sdk_configuration)
        self.knowledge = AsyncKnowledge(self.sdk_configuration)
        self.model = AsyncModel(self.sdk_configuration)
        self.tool_assisted_chat = AsyncToolAssistedChat(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List, Union

from typing_extensions import TypeAliasType, TypedDict

from friendli.types import BaseModel

from .b64imageitem import B64ImageItem, B64ImageItemTypedDict
from .urlimageitem import URLImageItem, URLImageItemTypedDict

DataTypedDict = TypeAliasType(
    "DataTypedDict", Union[List[URLImageItemTypedDict], List[B64ImageItemTypedDict]]
)


Data = TypeAliasType("Data", Union[List[URLImageItem], List[B64ImageItem]])


class DedicatedImageGenerateSuccessTypedDict(TypedDict):
    data: DataTypedDict


class DedicatedImageGenerateSuccess(BaseModel):
    data: Data



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class AcceleratorRequirementTypedDict(TypedDict):
    r"""Accelerator requirement."""

    count: int
    r"""Specifies the number of instances."""
    type: str
    r"""Specifies the instance type."""


class AcceleratorRequirement(BaseModel):
    r"""Accelerator requirement."""

    count: int
    r"""Specifies the number of instances."""

    type: str
    r"""Specifies the instance type."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const

from .function import Function, FunctionTypedDict


class ToolTypedDict(TypedDict):
    function: FunctionTypedDict
    type: Literal["function"]
    r"""The type of the tool. Currently, only `function` is supported."""


class Tool(BaseModel):
    function: Function

    TYPE: Annotated[
        Annotated[Literal["function"], AfterValidator(validate_const("function"))],
        pydantic.Field(alias="type"),
    ] = "function"
    r"""The type of the tool. Currently, only `function` is supported."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Union

from typing_extensions import TypeAliasType

from .completionsdedicatedmodelbodywithprompt import (
    CompletionsDedicatedModelBodyWithPrompt,
    CompletionsDedicatedModelBodyWithPromptTypedDict,
)
from .completionsdedicatedmodelbodywithtokens import (
    CompletionsDedicatedModelBodyWithTokens,
    CompletionsDedicatedModelBodyWithTokensTypedDict,
)

DedicatedCompletionsBodyTypedDict = TypeAliasType(
    "DedicatedCompletionsBodyTypedDict",
    Union[
        CompletionsDedicatedModelBodyWithPromptTypedDict,
        CompletionsDedicatedModelBodyWithTokensTypedDict,
    ],
)


DedicatedCompletionsBody = TypeAliasType(
    "DedicatedCompletionsBody",
    Union[
        CompletionsDedicatedModelBodyWithPrompt, CompletionsDedicatedModelBodyWithTokens
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List

from typing_extensions import TypedDict

from friendli.types import BaseModel

from .modelcatalogresponseitem import (
    ModelCatalogResponseItem,
    ModelCatalogResponseItemTypedDict,
)


class ServerlessModelListSuccessTypedDict(TypedDict):
    r"""`/models` API response (for OpenRouters)."""

    data: List[ModelCatalogResponseItemTypedDict]


class ServerlessModelListSuccess(BaseModel):
    r"""`/models` API response (for OpenRouters)."""

    data: List[ModelCatalogResponseItem]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const

from .responseformatjsonschemaschema import (
    ResponseFormatJSONSchemaSchema,
    ResponseFormatJSONSchemaSchemaTypedDict,
)


class ResponseFormatJSONSchemaTypedDict(TypedDict):
    json_schema: ResponseFormatJSONSchemaSchemaTypedDict
    type: Literal["json_schema"]
    r"""The type of the response format: `json_schema`"""


class ResponseFormatJSONSchema(BaseModel):
    json_schema: ResponseFormatJSONSchemaSchema

    TYPE: Annotated[
        Annotated[
            Literal["json_schema"], AfterValidator(validate_const("json_schema"))
        ],
        pydantic.Field(alias="type"),
    ] = "json_schema"
    r"""The type of the response format: `json_schema`"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

import pydantic
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel


class SchemaTypedDict(TypedDict):
    r"""The schema for the response format, described as a JSON Schema object."""


class Schema(BaseModel):
    r"""The schema for the response format, described as a JSON Schema object."""


class ResponseFormatJSONSchemaSchemaTypedDict(TypedDict):
    schema_: SchemaTypedDict
    r"""The schema for the response format, described as a JSON Schema object."""


class ResponseFormatJSONSchemaSchema(BaseModel):
    schema_: Annotated[Schema, pydantic.Field(alias="schema")]
    r"""The schema for the response format, described as a JSON Schema object."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List

from typing_extensions import TypedDict

from friendli.types import BaseModel


class ServerlessDetokenizationBodyTypedDict(TypedDict):
    model: str
    r"""Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models)."""
    tokens: List[int]
    r"""A token sequence to detokenize."""


class ServerlessDetokenizationBody(BaseModel):
    model: str
    r"""Code of the model to use. See [available model list](https://friendli.ai/docs/guides/serverless_endpoints/pricing#text-generation-models)."""

    tokens: List[int]
    r"""A token sequence to detokenize."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import List

from typing_extensions import TypedDict

from friendli.types import BaseModel


class DedicatedDetokenizationBodyTypedDict(TypedDict):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, use the format \"YOUR_ENDPOINT_ID:YOUR_ADAPTER_ROUTE\". Otherwise, you can just use \"YOUR_ENDPOINT_ID\" alone."""
    tokens: List[int]
    r"""A token sequence to detokenize."""


class DedicatedDetokenizationBody(BaseModel):
    model: str
    r"""ID of target endpoint. If you want to send request to specific adapter, use the format \"YOUR_ENDPOINT_ID:YOUR_ADAPTER_ROUTE\". Otherwise, you can just use \"YOUR_ENDPOINT_ID\" alone."""

    tokens: List[int]
    r"""A token sequence to detokenize."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const


class ResponseFormatJSONObjectTypedDict(TypedDict):
    type: Literal["json_object"]
    r"""The type of the response format: `json_object`"""


class ResponseFormatJSONObject(BaseModel):
    TYPE: Annotated[
        Annotated[
            Literal["json_object"], AfterValidator(validate_const("json_object"))
        ],
        pydantic.Field(alias="type"),
    ] = "json_object"
    r"""The type of the response format: `json_object`"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .acceleratorrequirement import (
    AcceleratorRequirement,
    AcceleratorRequirementTypedDict,
)
from .assistantmessage import AssistantMessage, AssistantMessageTypedDict
from .assistantmessagetoolcall import (
    AssistantMessageToolCall,
    AssistantMessageToolCallTypedDict,
)
from .assistantmessagetoolcallfunction import (
    AssistantMessageToolCallFunction,
    AssistantMessageToolCallFunctionTypedDict,
)
from .audiocontent import AudioContent, AudioContentTypedDict
from .audiodata import AudioData, AudioDataTypedDict
from .autoscalingpolicy import AutoscalingPolicy, AutoscalingPolicyTypedDict
from .b64imageitem import (
    B64ImageItem,
    B64ImageItemResponseFormat,
    B64ImageItemTypedDict,
)
from .chatchoice import ChatChoice, ChatChoiceTypedDict, FinishReason
from .chatchoicemessage import ChatChoiceMessage, ChatChoiceMessageTypedDict
from .chatcompletebodytoolchoice import (
    ChatCompleteBodyToolChoice,
    ChatCompleteBodyToolChoiceTypedDict,
)
from .chatcompletebodytoolchoicefunction import (
    ChatCompleteBodyToolChoiceFunction,
    ChatCompleteBodyToolChoiceFunctionTypedDict,
)
from .chatlogprobs import ChatLogprobs, ChatLogprobsTypedDict
from .chatlogprobscontent import ChatLogprobsContent, ChatLogprobsContentTypedDict
from .chatlogprobscontenttoplogprob import (
    ChatLogprobsContentTopLogprob,
    ChatLogprobsContentTopLogprobTypedDict,
)
from .chatusage import ChatUsage, ChatUsageTypedDict
from .completionsbodywithprompt import (
    CompletionsBodyWithPrompt,
    CompletionsBodyWithPromptPrompt,
    CompletionsBodyWithPromptPromptTypedDict,
    CompletionsBodyWithPromptSeed,
    CompletionsBodyWithPromptSeedTypedDict,
    CompletionsBodyWithPromptTypedDict,
)
from .completionsbodywithtokens import (
    CompletionsBodyWithTokens,
    CompletionsBodyWithTokensSeed,
    CompletionsBodyWithTokensSeedTypedDict,
    CompletionsBodyWithTokensTypedDict,
)
from .completionschoice import (
    CompletionsChoice,
    CompletionsChoiceFinishReason,
    CompletionsChoiceTypedDict,
)
from .completionsdedicatedmodelbodywithprompt import (
    CompletionsDedicatedModelBodyWithPrompt,
    CompletionsDedicatedModelBodyWithPromptPrompt,
    CompletionsDedicatedModelBodyWithPromptPromptTypedDict,
    CompletionsDedicatedModelBodyWithPromptSeed,
    CompletionsDedicatedModelBodyWithPromptSeedTypedDict,
    CompletionsDedicatedModelBodyWithPromptTypedDict,
)
from .completionsdedicatedmodelbodywithtokens import (
    CompletionsDedicatedModelBodyWithTokens,
    CompletionsDedicatedModelBodyWithTokensSeed,
    CompletionsDedicatedModelBodyWithTokensSeedTypedDict,
    CompletionsDedicatedModelBodyWithTokensTypedDict,
)
from .completionslogprobs import (
    CompletionsLogprobs,
    CompletionsLogprobsTypedDict,
    TopLogprobs,
    TopLogprobsTypedDict,
)
from .completionsserverlessmodelbodywithprompt import (
    CompletionsServerlessModelBodyWithPrompt,
    CompletionsServerlessModelBodyWithPromptSeed,
    CompletionsServerlessModelBodyWithPromptSeedTypedDict,
    CompletionsServerlessModelBodyWithPromptTypedDict,
    Prompt,
    PromptTypedDict,
)
from .completionsserverlessmodelbodywithtokens import (
    CompletionsServerlessModelBodyWithTokens,
    CompletionsServerlessModelBodyWithTokensSeed,
    CompletionsServerlessModelBodyWithTokensSeedTypedDict,
    CompletionsServerlessModelBodyWithTokensTypedDict,
)
from .completionsstreambodywithprompt import (
    CompletionsStreamBodyWithPrompt,
    CompletionsStreamBodyWithPromptPrompt,
    CompletionsStreamBodyWithPromptPromptTypedDict,
    CompletionsStreamBodyWithPromptSeed,
    CompletionsStreamBodyWithPromptSeedTypedDict,
    CompletionsStreamBodyWithPromptTypedDict,
)
from .completionsstreambodywithtokens import (
    CompletionsStreamBodyWithTokens,
    CompletionsStreamBodyWithTokensSeed,
    CompletionsStreamBodyWithTokensSeedTypedDict,
    CompletionsStreamBodyWithTokensTypedDict,
)
from .completionsstreamdedicatedmodelbodywithprompt import (
    CompletionsStreamDedicatedModelBodyWithPrompt,
    CompletionsStreamDedicatedModelBodyWithPromptPrompt,
    CompletionsStreamDedicatedModelBodyWithPromptPromptTypedDict,
    CompletionsStreamDedicatedModelBodyWithPromptSeed,
    CompletionsStreamDedicatedModelBodyWithPromptSeedTypedDict,
    CompletionsStreamDedicatedModelBodyWithPromptTypedDict,
)
from .completionsstreamdedicatedmodelbodywithtokens import (
    CompletionsStreamDedicatedModelBodyWithTokens,
    CompletionsStreamDedicatedModelBodyWithTokensSeed,
    CompletionsStreamDedicatedModelBodyWithTokensSeedTypedDict,
    CompletionsStreamDedicatedModelBodyWithTokensTypedDict,
)
from .completionsstreamserverlessmodelbodywithprompt import (
    CompletionsStreamServerlessModelBodyWithPrompt,
    CompletionsStreamServerlessModelBodyWithPromptPrompt,
    CompletionsStreamServerlessModelBodyWithPromptPromptTypedDict,
    CompletionsStreamServerlessModelBodyWithPromptSeed,
    CompletionsStreamServerlessModelBodyWithPromptSeedTypedDict,
    CompletionsStreamServerlessModelBodyWithPromptTypedDict,
)
from .completionsstreamserverlessmodelbodywithtokens import (
    CompletionsStreamServerlessModelBodyWithTokens,
    CompletionsStreamServerlessModelBodyWithTokensSeed,
    CompletionsStreamServerlessModelBodyWithTokensSeedTypedDict,
    CompletionsStreamServerlessModelBodyWithTokensTypedDict,
)
from .containerchatcompleteop import CONTAINER_CHAT_COMPLETE_OP_SERVERS
from .containerchatcompletesuccess import (
    ContainerChatCompleteSuccess,
    ContainerChatCompleteSuccessTypedDict,
)
from .containerchatcompletionbody import (
    ContainerChatCompletionBody,
    ContainerChatCompletionBodyLogitBias,
    ContainerChatCompletionBodyLogitBiasTypedDict,
    ContainerChatCompletionBodySeed,
    ContainerChatCompletionBodySeedTypedDict,
    ContainerChatCompletionBodyToolChoice,
    ContainerChatCompletionBodyToolChoiceTypedDict,
    ContainerChatCompletionBodyTypedDict,
)
from .containerchatcompletionstreambody import (
    ContainerChatCompletionStreamBody,
    ContainerChatCompletionStreamBodyLogitBias,
    ContainerChatCompletionStreamBodyLogitBiasTypedDict,
    ContainerChatCompletionStreamBodySeed,
    ContainerChatCompletionStreamBodySeedTypedDict,
    ContainerChatCompletionStreamBodyToolChoice,
    ContainerChatCompletionStreamBodyToolChoiceTypedDict,
    ContainerChatCompletionStreamBodyTypedDict,
)
from .containerchatcompletionstreamsuccess import (
    ContainerChatCompletionStreamSuccess,
    ContainerChatCompletionStreamSuccessTypedDict,
)
from .containerchatstreamop import CONTAINER_CHAT_STREAM_OP_SERVERS
from .containercompletionsbody import (
    ContainerCompletionsBody,
    ContainerCompletionsBodyTypedDict,
)
from .containercompletionscompleteop import CONTAINER_COMPLETIONS_COMPLETE_OP_SERVERS
from .containercompletionsstreambody import (
    ContainerCompletionsStreamBody,
    ContainerCompletionsStreamBodyTypedDict,
)
from .containercompletionsstreamop import CONTAINER_COMPLETIONS_STREAM_OP_SERVERS
from .containercompletionsstreamsuccess import (
    ContainerCompletionsStreamSuccess,
    ContainerCompletionsStreamSuccessTypedDict,
)
from .containercompletionssuccess import (
    ContainerCompletionsSuccess,
    ContainerCompletionsSuccessTypedDict,
)
from .containerdetokenizationbody import (
    ContainerDetokenizationBody,
    ContainerDetokenizationBodyTypedDict,
)
from .containerdetokenizationop import CONTAINER_DETOKENIZATION_OP_SERVERS
from .containerdetokenizationsuccess import (
    ContainerDetokenizationSuccess,
    ContainerDetokenizationSuccessTypedDict,
)
from .containertokenizationbody import (
    ContainerTokenizationBody,
    ContainerTokenizationBodyTypedDict,
)
from .containertokenizationop import CONTAINER_TOKENIZATION_OP_SERVERS
from .containertokenizationsuccess import (
    ContainerTokenizationSuccess,
    ContainerTokenizationSuccessTypedDict,
)
from .dedicatedchatcompleteop import (
    DedicatedChatCompleteRequest,
    DedicatedChatCompleteRequestTypedDict,
)
from .dedicatedchatcompletesuccess import (
    DedicatedChatCompleteSuccess,
    DedicatedChatCompleteSuccessTypedDict,
)
from .dedicatedchatcompletionbody import (
    DedicatedChatCompletionBody,
    DedicatedChatCompletionBodyLogitBias,
    DedicatedChatCompletionBodyLogitBiasTypedDict,
    DedicatedChatCompletionBodySeed,
    DedicatedChatCompletionBodySeedTypedDict,
    DedicatedChatCompletionBodyToolChoice,
    DedicatedChatCompletionBodyToolChoiceTypedDict,
    DedicatedChatCompletionBodyTypedDict,
)
from .dedicatedchatcompletionstreambody import (
    DedicatedChatCompletionStreamBody,
    DedicatedChatCompletionStreamBodyLogitBias,
    DedicatedChatCompletionStreamBodyLogitBiasTypedDict,
    DedicatedChatCompletionStreamBodySeed,
    DedicatedChatCompletionStreamBodySeedTypedDict,
    DedicatedChatCompletionStreamBodyToolChoice,
    DedicatedChatCompletionStreamBodyToolChoiceTypedDict,
    DedicatedChatCompletionStreamBodyTypedDict,
)
from .dedicatedchatcompletionstreamsuccess import (
    DedicatedChatCompletionStreamSuccess,
    DedicatedChatCompletionStreamSuccessTypedDict,
)
from .dedicatedchatstreamop import (
    DedicatedChatStreamRequest,
    DedicatedChatStreamRequestTypedDict,
)
from .dedicatedcompletionsbody import (
    DedicatedCompletionsBody,
    DedicatedCompletionsBodyTypedDict,
)
from .dedicatedcompletionscompleteop import (
    DedicatedCompletionsCompleteRequest,
    DedicatedCompletionsCompleteRequestTypedDict,
)
from .dedicatedcompletionsstreambody import (
    DedicatedCompletionsStreamBody,
    DedicatedCompletionsStreamBodyTypedDict,
)
from .dedicatedcompletionsstreamop import (
    DedicatedCompletionsStreamRequest,
    DedicatedCompletionsStreamRequestTypedDict,
)
from .dedicatedcompletionsstreamsuccess import (
    DedicatedCompletionsStreamSuccess,
    DedicatedCompletionsStreamSuccessTypedDict,
)
from .dedicatedcompletionssuccess import (
    DedicatedCompletionsSuccess,
    DedicatedCompletionsSuccessTypedDict,
)
from .dedicateddetokenizationbody import (
    DedicatedDetokenizationBody,
    DedicatedDetokenizationBodyTypedDict,
)
from .dedicateddetokenizationop import (
    DedicatedDetokenizationRequest,
    DedicatedDetokenizationRequestTypedDict,
)
from .dedicateddetokenizationsuccess import (
    DedicatedDetokenizationSuccess,
    DedicatedDetokenizationSuccessTypedDict,
)
from .dedicatedendpointwandbartifactcreatebody import (
    DedicatedEndpointWandbArtifactCreateBody,
    DedicatedEndpointWandbArtifactCreateBodyTypedDict,
)
from .dedicatedendpointwandbartifactcreateop import (
    DedicatedEndpointWandbArtifactCreateRequest,
    DedicatedEndpointWandbArtifactCreateRequestTypedDict,
)
from .dedicatedendpointwandbartifactcreateresponse import (
    DedicatedEndpointWandbArtifactCreateResponse,
    DedicatedEndpointWandbArtifactCreateResponseTypedDict,
)
from .dedicatedimagegeneratesuccess import (
    Data,
    DataTypedDict,
    DedicatedImageGenerateSuccess,
    DedicatedImageGenerateSuccessTypedDict,
)
from .dedicatedimagegenerationbody import (
    DedicatedImageGenerationBody,
    DedicatedImageGenerationBodyResponseFormat,
    DedicatedImageGenerationBodyTypedDict,
)
from .dedicatedimagesgenerateop import (
    DedicatedImagesGenerateRequest,
    DedicatedImagesGenerateRequestTypedDict,
)
from .dedicatedtokenizationbody import (
    DedicatedTokenizationBody,
    DedicatedTokenizationBodyTypedDict,
)
from .dedicatedtokenizationop import (
    DedicatedTokenizationRequest,
    DedicatedTokenizationRequestTypedDict,
)
from .dedicatedtokenizationsuccess import (
    DedicatedTokenizationSuccess,
    DedicatedTokenizationSuccessTypedDict,
)
from .filebuiltintool import FileBuiltInTool, FileBuiltInToolTypedDict
from .function import Function, FunctionTypedDict, Parameters, ParametersTypedDict
from .functionality import Functionality, FunctionalityTypedDict
from .functionresult import FunctionResult, FunctionResultTypedDict
from .imagecontent import ImageContent, ImageContentTypedDict
from .imagedata import ImageData, ImageDataTypedDict
from .knowledgeretrievedchunk import (
    KnowledgeRetrievedChunk,
    KnowledgeRetrievedChunkTypedDict,
)
from .message import Message, MessageTypedDict
from .modelcatalogresponseitem import (
    ModelCatalogResponseItem,
    ModelCatalogResponseItemTypedDict,
)
from .otherbuiltintool import OtherBuiltInTool, OtherBuiltInToolTypedDict, Type
from .pricingmodel import PricingModel, PricingModelTypedDict
from .responseformat import ResponseFormat, ResponseFormatTypedDict
from .responseformatjsonobject import (
    ResponseFormatJSONObject,
    ResponseFormatJSONObjectTypedDict,
)
from .responseformatjsonschema import (
    ResponseFormatJSONSchema,
    ResponseFormatJSONSchemaTypedDict,
)
from .responseformatjsonschemaschema import (
    ResponseFormatJSONSchemaSchema,
    ResponseFormatJSONSchemaSchemaTypedDict,
    Schema,
    SchemaTypedDict,
)
from .responseformatregex import ResponseFormatRegex, ResponseFormatRegexTypedDict
from .responseformattext import ResponseFormatText, ResponseFormatTextTypedDict
from .sdkerror import SDKError
from .security import Security, SecurityTypedDict
from .serverlesschatcompleteop import (
    ServerlessChatCompleteRequest,
    ServerlessChatCompleteRequestTypedDict,
)
from .serverlesschatcompletesuccess import (
    ServerlessChatCompleteSuccess,
    ServerlessChatCompleteSuccessTypedDict,
)
from .serverlesschatcompletionbody import (
    LogitBias,
    LogitBiasTypedDict,
    Seed,
    SeedTypedDict,
    ServerlessChatCompletionBody,
    ServerlessChatCompletionBodyTypedDict,
    ToolChoice,
    ToolChoiceTypedDict,
)
from .serverlesschatcompletionstreambody import (
    ServerlessChatCompletionStreamBody,
    ServerlessChatCompletionStreamBodyLogitBias,
    ServerlessChatCompletionStreamBodyLogitBiasTypedDict,
    ServerlessChatCompletionStreamBodySeed,
    ServerlessChatCompletionStreamBodySeedTypedDict,
    ServerlessChatCompletionStreamBodyToolChoice,
    ServerlessChatCompletionStreamBodyToolChoiceTypedDict,
    ServerlessChatCompletionStreamBodyTypedDict,
)
from .serverlesschatcompletionstreamsuccess import (
    ServerlessChatCompletionStreamSuccess,
    ServerlessChatCompletionStreamSuccessTypedDict,
)
from .serverlesschatstreamop import (
    ServerlessChatStreamRequest,
    ServerlessChatStreamRequestTypedDict,
)
from .serverlesscompletionsbody import (
    ServerlessCompletionsBody,
    ServerlessCompletionsBodyTypedDict,
)
from .serverlesscompletionscompleteop import (
    ServerlessCompletionsCompleteRequest,
    ServerlessCompletionsCompleteRequestTypedDict,
)
from .serverlesscompletionsstreambody import (
    ServerlessCompletionsStreamBody,
    ServerlessCompletionsStreamBodyTypedDict,
)
from .serverlesscompletionsstreamop import (
    ServerlessCompletionsStreamRequest,
    ServerlessCompletionsStreamRequestTypedDict,
)
from .serverlesscompletionsstreamsuccess import (
    ServerlessCompletionsStreamSuccess,
    ServerlessCompletionsStreamSuccessTypedDict,
)
from .serverlesscompletionssuccess import (
    ServerlessCompletionsSuccess,
    ServerlessCompletionsSuccessTypedDict,
)
from .serverlessdetokenizationbody import (
    ServerlessDetokenizationBody,
    ServerlessDetokenizationBodyTypedDict,
)
from .serverlessdetokenizationop import (
    ServerlessDetokenizationRequest,
    ServerlessDetokenizationRequestTypedDict,
)
from .serverlessdetokenizationsuccess import (
    ServerlessDetokenizationSuccess,
    ServerlessDetokenizationSuccessTypedDict,
)
from .serverlessknowledgeretrievalbody import (
    ServerlessKnowledgeRetrievalBody,
    ServerlessKnowledgeRetrievalBodyTypedDict,
)
from .serverlessknowledgeretrievalsuccess import (
    ServerlessKnowledgeRetrievalSuccess,
    ServerlessKnowledgeRetrievalSuccessTypedDict,
)
from .serverlessknowledgeretrieveop import (
    ServerlessKnowledgeRetrieveRequest,
    ServerlessKnowledgeRetrieveRequestTypedDict,
)
from .serverlessmodellistsuccess import (
    ServerlessModelListSuccess,
    ServerlessModelListSuccessTypedDict,
)
from .serverlesstokenizationbody import (
    ServerlessTokenizationBody,
    ServerlessTokenizationBodyTypedDict,
)
from .serverlesstokenizationop import (
    ServerlessTokenizationRequest,
    ServerlessTokenizationRequestTypedDict,
)
from .serverlesstokenizationsuccess import (
    ServerlessTokenizationSuccess,
    ServerlessTokenizationSuccessTypedDict,
)
from .serverlesstoolassistedchatcompleteop import (
    ServerlessToolAssistedChatCompleteRequest,
    ServerlessToolAssistedChatCompleteRequestTypedDict,
)
from .serverlesstoolassistedchatcompletesuccess import (
    ServerlessToolAssistedChatCompleteSuccess,
    ServerlessToolAssistedChatCompleteSuccessTypedDict,
)
from .serverlesstoolassistedchatcompletionbody import (
    ServerlessToolAssistedChatCompletionBody,
    ServerlessToolAssistedChatCompletionBodyLogitBias,
    ServerlessToolAssistedChatCompletionBodyLogitBiasTypedDict,
    ServerlessToolAssistedChatCompletionBodySeed,
    ServerlessToolAssistedChatCompletionBodySeedTypedDict,
    ServerlessToolAssistedChatCompletionBodyToolChoice,
    ServerlessToolAssistedChatCompletionBodyToolChoiceTypedDict,
    ServerlessToolAssistedChatCompletionBodyTypedDict,
)
from .serverlesstoolassistedchatcompletionstreambody import (
    ServerlessToolAssistedChatCompletionStreamBody,
    ServerlessToolAssistedChatCompletionStreamBodyLogitBias,
    ServerlessToolAssistedChatCompletionStreamBodyLogitBiasTypedDict,
    ServerlessToolAssistedChatCompletionStreamBodySeed,
    ServerlessToolAssistedChatCompletionStreamBodySeedTypedDict,
    ServerlessToolAssistedChatCompletionStreamBodyToolChoice,
    ServerlessToolAssistedChatCompletionStreamBodyToolChoiceTypedDict,
    ServerlessToolAssistedChatCompletionStreamBodyTypedDict,
)
from .serverlesstoolassistedchatcompletionstreamsuccess import (
    ServerlessToolAssistedChatCompletionStreamSuccess,
    ServerlessToolAssistedChatCompletionStreamSuccessTypedDict,
)
from .serverlesstoolassistedchatstreamop import (
    ServerlessToolAssistedChatStreamRequest,
    ServerlessToolAssistedChatStreamRequestTypedDict,
)
from .streamedchatchoice import (
    StreamedChatChoice,
    StreamedChatChoiceFinishReason,
    StreamedChatChoiceTypedDict,
)
from .streamedchatchoicedelta import (
    StreamedChatChoiceDelta,
    StreamedChatChoiceDeltaTypedDict,
)
from .streamedchatdata import StreamedChatData, StreamedChatDataTypedDict
from .streamedcompletiondata import (
    StreamedCompletionData,
    StreamedCompletionDataTypedDict,
)
from .streamedcompletionschoice import (
    StreamedCompletionsChoice,
    StreamedCompletionsChoiceFinishReason,
    StreamedCompletionsChoiceTypedDict,
)
from .streamedfunctionresult import (
    StreamedFunctionResult,
    StreamedFunctionResultTypedDict,
)
from .streamedtoolassistedchattoken import (
    StreamedToolAssistedChatToken,
    StreamedToolAssistedChatTokenTypedDict,
)
from .streamedtoolassistedchattoolstatus import (
    StreamedToolAssistedChatToolStatus,
    StreamedToolAssistedChatToolStatusTypedDict,
)
from .streamedtoolcallresult import (
    StreamedToolCallResult,
    StreamedToolCallResultTypedDict,
)
from .streamoptions import StreamOptions, StreamOptionsTypedDict
from .systemmessage import SystemMessage, SystemMessageTypedDict
from .textcontent import TextContent, TextContentTypedDict
from .tokensequence import TokenSequence, TokenSequenceTypedDict
from .tool import Tool, ToolTypedDict
from .toolassistedchattool import ToolAssistedChatTool, ToolAssistedChatToolTypedDict
from .toolcallresult import ToolCallResult, ToolCallResultTypedDict
from .toolmessage import ToolMessage, ToolMessageTypedDict
from .toolstatusdata import Name, Status, ToolStatusData, ToolStatusDataTypedDict
from .toolstatuserror import ToolStatusError, ToolStatusErrorTypedDict
from .toolstatusfile import ToolStatusFile, ToolStatusFileTypedDict
from .toolstatusparameter import ToolStatusParameter, ToolStatusParameterTypedDict
from .urlimageitem import (
    URLImageItem,
    URLImageItemResponseFormat,
    URLImageItemTypedDict,
)
from .usage import Usage, UsageTypedDict
from .usermessage import Content, ContentTypedDict, UserMessage, UserMessageTypedDict
from .usermessagecontentmultimodal import (
    UserMessageContentMultiModal,
    UserMessageContentMultiModalTypedDict,
)
from .videocontent import VideoContent, VideoContentTypedDict
from .videodata import VideoData, VideoDataTypedDict

__all__ = [
    "AcceleratorRequirement",
    "AcceleratorRequirementTypedDict",
    "AssistantMessage",
    "AssistantMessageToolCall",
    "AssistantMessageToolCallFunction",
    "AssistantMessageToolCallFunctionTypedDict",
    "AssistantMessageToolCallTypedDict",
    "AssistantMessageTypedDict",
    "AudioContent",
    "AudioContentTypedDict",
    "AudioData",
    "AudioDataTypedDict",
    "AutoscalingPolicy",
    "AutoscalingPolicyTypedDict",
    "B64ImageItem",
    "B64ImageItemResponseFormat",
    "B64ImageItemTypedDict",
    "CONTAINER_CHAT_COMPLETE_OP_SERVERS",
    "CONTAINER_CHAT_STREAM_OP_SERVERS",
    "CONTAINER_COMPLETIONS_COMPLETE_OP_SERVERS",
    "CONTAINER_COMPLETIONS_STREAM_OP_SERVERS",
    "CONTAINER_DETOKENIZATION_OP_SERVERS",
    "CONTAINER_TOKENIZATION_OP_SERVERS",
    "ChatChoice",
    "ChatChoiceMessage",
    "ChatChoiceMessageTypedDict",
    "ChatChoiceTypedDict",
    "ChatCompleteBodyToolChoice",
    "ChatCompleteBodyToolChoiceFunction",
    "ChatCompleteBodyToolChoiceFunctionTypedDict",
    "ChatCompleteBodyToolChoiceTypedDict",
    "ChatLogprobs",
    "ChatLogprobsContent",
    "ChatLogprobsContentTopLogprob",
    "ChatLogprobsContentTopLogprobTypedDict",
    "ChatLogprobsContentTypedDict",
    "ChatLogprobsTypedDict",
    "ChatUsage",
    "ChatUsageTypedDict",
    "CompletionsBodyWithPrompt",
    "CompletionsBodyWithPromptPrompt",
    "CompletionsBodyWithPromptPromptTypedDict",
    "CompletionsBodyWithPromptSeed",
    "CompletionsBodyWithPromptSeedTypedDict",
    "CompletionsBodyWithPromptTypedDict",
    "CompletionsBodyWithTokens",
    "CompletionsBodyWithTokensSeed",
    "CompletionsBodyWithTokensSeedTypedDict",
    "CompletionsBodyWithTokensTypedDict",
    "CompletionsChoice",
    "CompletionsChoiceFinishReason",
    "CompletionsChoiceTypedDict",
    "CompletionsDedicatedModelBodyWithPrompt",
    "CompletionsDedicatedModelBodyWithPromptPrompt",
    "CompletionsDedicatedModelBodyWithPromptPromptTypedDict",
    "CompletionsDedicatedModelBodyWithPromptSeed",
    "CompletionsDedicatedModelBodyWithPromptSeedTypedDict",
    "CompletionsDedicatedModelBodyWithPromptTypedDict",
    "CompletionsDedicatedModelBodyWithTokens",
    "CompletionsDedicatedModelBodyWithTokensSeed",
    "CompletionsDedicatedModelBodyWithTokensSeedTypedDict",
    "CompletionsDedicatedModelBodyWithTokensTypedDict",
    "CompletionsLogprobs",
    "CompletionsLogprobsTypedDict",
    "CompletionsServerlessModelBodyWithPrompt",
    "CompletionsServerlessModelBodyWithPromptSeed",
    "CompletionsServerlessModelBodyWithPromptSeedTypedDict",
    "CompletionsServerlessModelBodyWithPromptTypedDict",
    "CompletionsServerlessModelBodyWithTokens",
    "CompletionsServerlessModelBodyWithTokensSeed",
    "CompletionsServerlessModelBodyWithTokensSeedTypedDict",
    "CompletionsServerlessModelBodyWithTokensTypedDict",
    "CompletionsStreamBodyWithPrompt",
    "CompletionsStreamBodyWithPromptPrompt",
    "CompletionsStreamBodyWithPromptPromptTypedDict",
    "CompletionsStreamBodyWithPromptSeed",
    "CompletionsStreamBodyWithPromptSeedTypedDict",
    "CompletionsStreamBodyWithPromptTypedDict",
    "CompletionsStreamBodyWithTokens",
    "CompletionsStreamBodyWithTokensSeed",
    "CompletionsStreamBodyWithTokensSeedTypedDict",
    "CompletionsStreamBodyWithTokensTypedDict",
    "CompletionsStreamDedicatedModelBodyWithPrompt",
    "CompletionsStreamDedicatedModelBodyWithPromptPrompt",
    "CompletionsStreamDedicatedModelBodyWithPromptPromptTypedDict",
    "CompletionsStreamDedicatedModelBodyWithPromptSeed",
    "CompletionsStreamDedicatedModelBodyWithPromptSeedTypedDict",
    "CompletionsStreamDedicatedModelBodyWithPromptTypedDict",
    "CompletionsStreamDedicatedModelBodyWithTokens",
    "CompletionsStreamDedicatedModelBodyWithTokensSeed",
    "CompletionsStreamDedicatedModelBodyWithTokensSeedTypedDict",
    "CompletionsStreamDedicatedModelBodyWithTokensTypedDict",
    "CompletionsStreamServerlessModelBodyWithPrompt",
    "CompletionsStreamServerlessModelBodyWithPromptPrompt",
    "CompletionsStreamServerlessModelBodyWithPromptPromptTypedDict",
    "CompletionsStreamServerlessModelBodyWithPromptSeed",
    "CompletionsStreamServerlessModelBodyWithPromptSeedTypedDict",
    "CompletionsStreamServerlessModelBodyWithPromptTypedDict",
    "CompletionsStreamServerlessModelBodyWithTokens",
    "CompletionsStreamServerlessModelBodyWithTokensSeed",
    "CompletionsStreamServerlessModelBodyWithTokensSeedTypedDict",
    "CompletionsStreamServerlessModelBodyWithTokensTypedDict",
    "ContainerChatCompleteSuccess",
    "ContainerChatCompleteSuccessTypedDict",
    "ContainerChatCompletionBody",
    "ContainerChatCompletionBodyLogitBias",
    "ContainerChatCompletionBodyLogitBiasTypedDict",
    "ContainerChatCompletionBodySeed",
    "ContainerChatCompletionBodySeedTypedDict",
    "ContainerChatCompletionBodyToolChoice",
    "ContainerChatCompletionBodyToolChoiceTypedDict",
    "ContainerChatCompletionBodyTypedDict",
    "ContainerChatCompletionStreamBody",
    "ContainerChatCompletionStreamBodyLogitBias",
    "ContainerChatCompletionStreamBodyLogitBiasTypedDict",
    "ContainerChatCompletionStreamBodySeed",
    "ContainerChatCompletionStreamBodySeedTypedDict",
    "ContainerChatCompletionStreamBodyToolChoice",
    "ContainerChatCompletionStreamBodyToolChoiceTypedDict",
    "ContainerChatCompletionStreamBodyTypedDict",
    "ContainerChatCompletionStreamSuccess",
    "ContainerChatCompletionStreamSuccessTypedDict",
    "ContainerCompletionsBody",
    "ContainerCompletionsBodyTypedDict",
    "ContainerCompletionsStreamBody",
    "ContainerCompletionsStreamBodyTypedDict",
    "ContainerCompletionsStreamSuccess",
    "ContainerCompletionsStreamSuccessTypedDict",
    "ContainerCompletionsSuccess",
    "ContainerCompletionsSuccessTypedDict",
    "ContainerDetokenizationBody",
    "ContainerDetokenizationBodyTypedDict",
    "ContainerDetokenizationSuccess",
    "ContainerDetokenizationSuccessTypedDict",
    "ContainerTokenizationBody",
    "ContainerTokenizationBodyTypedDict",
    "ContainerTokenizationSuccess",
    "ContainerTokenizationSuccessTypedDict",
    "Content",
    "ContentTypedDict",
    "Data",
    "DataTypedDict",
    "DedicatedChatCompleteRequest",
    "DedicatedChatCompleteRequestTypedDict",
    "DedicatedChatCompleteSuccess",
    "DedicatedChatCompleteSuccessTypedDict",
    "DedicatedChatCompletionBody",
    "DedicatedChatCompletionBodyLogitBias",
    "DedicatedChatCompletionBodyLogitBiasTypedDict",
    "DedicatedChatCompletionBodySeed",
    "DedicatedChatCompletionBodySeedTypedDict",
    "DedicatedChatCompletionBodyToolChoice",
    "DedicatedChatCompletionBodyToolChoiceTypedDict",
    "DedicatedChatCompletionBodyTypedDict",
    "DedicatedChatCompletionStreamBody",
    "DedicatedChatCompletionStreamBodyLogitBias",
    "DedicatedChatCompletionStreamBodyLogitBiasTypedDict",
    "DedicatedChatCompletionStreamBodySeed",
    "DedicatedChatCompletionStreamBodySeedTypedDict",
    "DedicatedChatCompletionStreamBodyToolChoice",
    "DedicatedChatCompletionStreamBodyToolChoiceTypedDict",
    "DedicatedChatCompletionStreamBodyTypedDict",
    "DedicatedChatCompletionStreamSuccess",
    "DedicatedChatCompletionStreamSuccessTypedDict",
    "DedicatedChatStreamRequest",
    "DedicatedChatStreamRequestTypedDict",
    "DedicatedCompletionsBody",
    "DedicatedCompletionsBodyTypedDict",
    "DedicatedCompletionsCompleteRequest",
    "DedicatedCompletionsCompleteRequestTypedDict",
    "DedicatedCompletionsStreamBody",
    "DedicatedCompletionsStreamBodyTypedDict",
    "DedicatedCompletionsStreamRequest",
    "DedicatedCompletionsStreamRequestTypedDict",
    "DedicatedCompletionsStreamSuccess",
    "DedicatedCompletionsStreamSuccessTypedDict",
    "DedicatedCompletionsSuccess",
    "DedicatedCompletionsSuccessTypedDict",
    "DedicatedDetokenizationBody",
    "DedicatedDetokenizationBodyTypedDict",
    "DedicatedDetokenizationRequest",
    "DedicatedDetokenizationRequestTypedDict",
    "DedicatedDetokenizationSuccess",
    "DedicatedDetokenizationSuccessTypedDict",
    "DedicatedEndpointWandbArtifactCreateBody",
    "DedicatedEndpointWandbArtifactCreateBodyTypedDict",
    "DedicatedEndpointWandbArtifactCreateRequest",
    "DedicatedEndpointWandbArtifactCreateRequestTypedDict",
    "DedicatedEndpointWandbArtifactCreateResponse",
    "DedicatedEndpointWandbArtifactCreateResponseTypedDict",
    "DedicatedImageGenerateSuccess",
    "DedicatedImageGenerateSuccessTypedDict",
    "DedicatedImageGenerationBody",
    "DedicatedImageGenerationBodyResponseFormat",
    "DedicatedImageGenerationBodyTypedDict",
    "DedicatedImagesGenerateRequest",
    "DedicatedImagesGenerateRequestTypedDict",
    "DedicatedTokenizationBody",
    "DedicatedTokenizationBodyTypedDict",
    "DedicatedTokenizationRequest",
    "DedicatedTokenizationRequestTypedDict",
    "DedicatedTokenizationSuccess",
    "DedicatedTokenizationSuccessTypedDict",
    "FileBuiltInTool",
    "FileBuiltInToolTypedDict",
    "FinishReason",
    "Function",
    "FunctionResult",
    "FunctionResultTypedDict",
    "FunctionTypedDict",
    "Functionality",
    "FunctionalityTypedDict",
    "ImageContent",
    "ImageContentTypedDict",
    "ImageData",
    "ImageDataTypedDict",
    "KnowledgeRetrievedChunk",
    "KnowledgeRetrievedChunkTypedDict",
    "LogitBias",
    "LogitBiasTypedDict",
    "Message",
    "MessageTypedDict",
    "ModelCatalogResponseItem",
    "ModelCatalogResponseItemTypedDict",
    "Name",
    "OtherBuiltInTool",
    "OtherBuiltInToolTypedDict",
    "Parameters",
    "ParametersTypedDict",
    "PricingModel",
    "PricingModelTypedDict",
    "Prompt",
    "PromptTypedDict",
    "ResponseFormat",
    "ResponseFormatJSONObject",
    "ResponseFormatJSONObjectTypedDict",
    "ResponseFormatJSONSchema",
    "ResponseFormatJSONSchemaSchema",
    "ResponseFormatJSONSchemaSchemaTypedDict",
    "ResponseFormatJSONSchemaTypedDict",
    "ResponseFormatRegex",
    "ResponseFormatRegexTypedDict",
    "ResponseFormatText",
    "ResponseFormatTextTypedDict",
    "ResponseFormatTypedDict",
    "SDKError",
    "Schema",
    "SchemaTypedDict",
    "Security",
    "SecurityTypedDict",
    "Seed",
    "SeedTypedDict",
    "ServerlessChatCompleteRequest",
    "ServerlessChatCompleteRequestTypedDict",
    "ServerlessChatCompleteSuccess",
    "ServerlessChatCompleteSuccessTypedDict",
    "ServerlessChatCompletionBody",
    "ServerlessChatCompletionBodyTypedDict",
    "ServerlessChatCompletionStreamBody",
    "ServerlessChatCompletionStreamBodyLogitBias",
    "ServerlessChatCompletionStreamBodyLogitBiasTypedDict",
    "ServerlessChatCompletionStreamBodySeed",
    "ServerlessChatCompletionStreamBodySeedTypedDict",
    "ServerlessChatCompletionStreamBodyToolChoice",
    "ServerlessChatCompletionStreamBodyToolChoiceTypedDict",
    "ServerlessChatCompletionStreamBodyTypedDict",
    "ServerlessChatCompletionStreamSuccess",
    "ServerlessChatCompletionStreamSuccessTypedDict",
    "ServerlessChatStreamRequest",
    "ServerlessChatStreamRequestTypedDict",
    "ServerlessCompletionsBody",
    "ServerlessCompletionsBodyTypedDict",
    "ServerlessCompletionsCompleteRequest",
    "ServerlessCompletionsCompleteRequestTypedDict",
    "ServerlessCompletionsStreamBody",
    "ServerlessCompletionsStreamBodyTypedDict",
    "ServerlessCompletionsStreamRequest",
    "ServerlessCompletionsStreamRequestTypedDict",
    "ServerlessCompletionsStreamSuccess",
    "ServerlessCompletionsStreamSuccessTypedDict",
    "ServerlessCompletionsSuccess",
    "ServerlessCompletionsSuccessTypedDict",
    "ServerlessDetokenizationBody",
    "ServerlessDetokenizationBodyTypedDict",
    "ServerlessDetokenizationRequest",
    "ServerlessDetokenizationRequestTypedDict",
    "ServerlessDetokenizationSuccess",
    "ServerlessDetokenizationSuccessTypedDict",
    "ServerlessKnowledgeRetrievalBody",
    "ServerlessKnowledgeRetrievalBodyTypedDict",
    "ServerlessKnowledgeRetrievalSuccess",
    "ServerlessKnowledgeRetrievalSuccessTypedDict",
    "ServerlessKnowledgeRetrieveRequest",
    "ServerlessKnowledgeRetrieveRequestTypedDict",
    "ServerlessModelListSuccess",
    "ServerlessModelListSuccessTypedDict",
    "ServerlessTokenizationBody",
    "ServerlessTokenizationBodyTypedDict",
    "ServerlessTokenizationRequest",
    "ServerlessTokenizationRequestTypedDict",
    "ServerlessTokenizationSuccess",
    "ServerlessTokenizationSuccessTypedDict",
    "ServerlessToolAssistedChatCompleteRequest",
    "ServerlessToolAssistedChatCompleteRequestTypedDict",
    "ServerlessToolAssistedChatCompleteSuccess",
    "ServerlessToolAssistedChatCompleteSuccessTypedDict",
    "ServerlessToolAssistedChatCompletionBody",
    "ServerlessToolAssistedChatCompletionBodyLogitBias",
    "ServerlessToolAssistedChatCompletionBodyLogitBiasTypedDict",
    "ServerlessToolAssistedChatCompletionBodySeed",
    "ServerlessToolAssistedChatCompletionBodySeedTypedDict",
    "ServerlessToolAssistedChatCompletionBodyToolChoice",
    "ServerlessToolAssistedChatCompletionBodyToolChoiceTypedDict",
    "ServerlessToolAssistedChatCompletionBodyTypedDict",
    "ServerlessToolAssistedChatCompletionStreamBody",
    "ServerlessToolAssistedChatCompletionStreamBodyLogitBias",
    "ServerlessToolAssistedChatCompletionStreamBodyLogitBiasTypedDict",
    "ServerlessToolAssistedChatCompletionStreamBodySeed",
    "ServerlessToolAssistedChatCompletionStreamBodySeedTypedDict",
    "ServerlessToolAssistedChatCompletionStreamBodyToolChoice",
    "ServerlessToolAssistedChatCompletionStreamBodyToolChoiceTypedDict",
    "ServerlessToolAssistedChatCompletionStreamBodyTypedDict",
    "ServerlessToolAssistedChatCompletionStreamSuccess",
    "ServerlessToolAssistedChatCompletionStreamSuccessTypedDict",
    "ServerlessToolAssistedChatStreamRequest",
    "ServerlessToolAssistedChatStreamRequestTypedDict",
    "Status",
    "StreamOptions",
    "StreamOptionsTypedDict",
    "StreamedChatChoice",
    "StreamedChatChoiceDelta",
    "StreamedChatChoiceDeltaTypedDict",
    "StreamedChatChoiceFinishReason",
    "StreamedChatChoiceTypedDict",
    "StreamedChatData",
    "StreamedChatDataTypedDict",
    "StreamedCompletionData",
    "StreamedCompletionDataTypedDict",
    "StreamedCompletionsChoice",
    "StreamedCompletionsChoiceFinishReason",
    "StreamedCompletionsChoiceTypedDict",
    "StreamedFunctionResult",
    "StreamedFunctionResultTypedDict",
    "StreamedToolAssistedChatToken",
    "StreamedToolAssistedChatTokenTypedDict",
    "StreamedToolAssistedChatToolStatus",
    "StreamedToolAssistedChatToolStatusTypedDict",
    "StreamedToolCallResult",
    "StreamedToolCallResultTypedDict",
    "SystemMessage",
    "SystemMessageTypedDict",
    "TextContent",
    "TextContentTypedDict",
    "TokenSequence",
    "TokenSequenceTypedDict",
    "Tool",
    "ToolAssistedChatTool",
    "ToolAssistedChatToolTypedDict",
    "ToolCallResult",
    "ToolCallResultTypedDict",
    "ToolChoice",
    "ToolChoiceTypedDict",
    "ToolMessage",
    "ToolMessageTypedDict",
    "ToolStatusData",
    "ToolStatusDataTypedDict",
    "ToolStatusError",
    "ToolStatusErrorTypedDict",
    "ToolStatusFile",
    "ToolStatusFileTypedDict",
    "ToolStatusParameter",
    "ToolStatusParameterTypedDict",
    "ToolTypedDict",
    "TopLogprobs",
    "TopLogprobsTypedDict",
    "Type",
    "URLImageItem",
    "URLImageItemResponseFormat",
    "URLImageItemTypedDict",
    "Usage",
    "UsageTypedDict",
    "UserMessage",
    "UserMessageContentMultiModal",
    "UserMessageContentMultiModalTypedDict",
    "UserMessageTypedDict",
    "VideoContent",
    "VideoContentTypedDict",
    "VideoData",
    "VideoDataTypedDict",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable


class StreamedFunctionResultTypedDict(TypedDict):
    arguments: str
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """
    name: NotRequired[Nullable[str]]
    r"""The name of the function to call."""


class StreamedFunctionResult(BaseModel):
    arguments: str
    r"""The arguments for calling the function, generated by the model in JSON format.
    Ensure to validate these arguments in your code before invoking the function since the model may not always produce valid JSON.

    """

    name: OptionalNullable[str] = UNSET
    r"""The name of the function to call."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["name"]
        nullable_fields = ["name"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from decimal import Decimal
from typing import (
    Any,
    Dict,
    List,
    Optional,
    Union,
    get_args,
    get_origin,
    get_type_hints,
)

from pydantic import BaseModel
from pydantic.fields import FieldInfo

from .metadata import (
    PathParamMetadata,
    find_field_metadata,
)
from .values import (
    _get_serialized_params,
    _is_set,
    _populate_from_globals,
    _val_to_string,
)


def generate_url(
    server_url: str,
    path: str,
    path_params: Any,
    gbls: Optional[Any] = None,
) -> str:
    path_param_values: Dict[str, str] = {}

    globals_already_populated = _populate_path_params(
        path_params, gbls, path_param_values, []
    )
    if _is_set(gbls):
        _populate_path_params(gbls, None, path_param_values, globals_already_populated)

    for key, value in path_param_values.items():
        path = path.replace("{" + key + "}", value, 1)

    return remove_suffix(server_url, "/") + path


def _populate_path_params(
    path_params: Any,
    gbls: Any,
    path_param_values: Dict[str, str],
    skip_fields: List[str],
) -> List[str]:
    globals_already_populated: List[str] = []

    if not isinstance(path_params, BaseModel):
        return globals_already_populated

    path_param_fields: Dict[str, FieldInfo] = path_params.__class__.model_fields
    path_param_field_types = get_type_hints(path_params.__class__)
    for name in path_param_fields:
        if name in skip_fields:
            continue

        field = path_param_fields[name]

        param_metadata = find_field_metadata(field, PathParamMetadata)
        if param_metadata is None:
            continue

        param = getattr(path_params, name) if _is_set(path_params) else None
        param, global_found = _populate_from_globals(
            name, param, PathParamMetadata, gbls
        )
        if global_found:
            globals_already_populated.append(name)

        if not _is_set(param):
            continue

        f_name = field.alias if field.alias is not None else name
        serialization = param_metadata.serialization
        if serialization is not None:
            serialized_params = _get_serialized_params(
                param_metadata, f_name, param, path_param_field_types[name]
            )
            for key, value in serialized_params.items():
                path_param_values[key] = value
        else:
            pp_vals: List[str] = []
            if param_metadata.style == "simple":
                if isinstance(param, List):
                    for pp_val in param:
                        if not _is_set(pp_val):
                            continue
                        pp_vals.append(_val_to_string(pp_val))
                    path_param_values[f_name] = ",".join(pp_vals)
                elif isinstance(param, Dict):
                    for pp_key in param:
                        if not _is_set(param[pp_key]):
                            continue
                        if param_metadata.explode:
                            pp_vals.append(f"{pp_key}={_val_to_string(param[pp_key])}")
                        else:
                            pp_vals.append(f"{pp_key},{_val_to_string(param[pp_key])}")
                    path_param_values[f_name] = ",".join(pp_vals)
                elif not isinstance(param, (str, int, float, complex, bool, Decimal)):
                    param_fields: Dict[str, FieldInfo] = param.__class__.model_fields
                    for name in param_fields:
                        param_field = param_fields[name]

                        param_value_metadata = find_field_metadata(
                            param_field, PathParamMetadata
                        )
                        if param_value_metadata is None:
                            continue

                        param_name = (
                            param_field.alias if param_field.alias is not None else name
                        )

                        param_field_val = getattr(param, name)
                        if not _is_set(param_field_val):
                            continue
                        if param_metadata.explode:
                            pp_vals.append(
                                f"{param_name}={_val_to_string(param_field_val)}"
                            )
                        else:
                            pp_vals.append(
                                f"{param_name},{_val_to_string(param_field_val)}"
                            )
                    path_param_values[f_name] = ",".join(pp_vals)
                elif _is_set(param):
                    path_param_values[f_name] = _val_to_string(param)

    return globals_already_populated


def is_optional(field):
    return get_origin(field) is Union and type(None) in get_args(field)


def template_url(url_with_params: str, params: Dict[str, str]) -> str:
    for key, value in params.items():
        url_with_params = url_with_params.replace("{" + key + "}", value)

    return url_with_params


def remove_suffix(input_string, suffix):
    if suffix and input_string.endswith(suffix):
        return input_string[: -len(suffix)]
    return input_string



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from abc import abstractmethod

from friendli.endpoint import AsyncEndpoint, SyncEndpoint
from friendli.friendli_chat import AsyncFriendliChat, SyncFriendliChat
from friendli.friendli_completions import (
    AsyncFriendliCompletions,
    SyncFriendliCompletions,
)
from friendli.friendli_token import AsyncFriendliToken, SyncFriendliToken
from friendli.image import AsyncImage, SyncImage

from .basesdk import AsyncSDK, BaseSDK, SyncSDK
from .sdkconfiguration import SDKConfiguration


class BaseDedicated(BaseSDK):
    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    @abstractmethod
    def _init_sdks(self):
        pass


class SyncDedicated(BaseDedicated, SyncSDK):
    chat: SyncFriendliChat
    completions: SyncFriendliCompletions
    token: SyncFriendliToken
    image: SyncImage
    endpoint: SyncEndpoint

    def _init_sdks(self):
        self.chat = SyncFriendliChat(self.sdk_configuration)
        self.completions = SyncFriendliCompletions(self.sdk_configuration)
        self.token = SyncFriendliToken(self.sdk_configuration)
        self.image = SyncImage(self.sdk_configuration)
        self.endpoint = SyncEndpoint(self.sdk_configuration)


class AsyncDedicated(BaseDedicated, AsyncSDK):
    chat: AsyncFriendliChat
    completions: AsyncFriendliCompletions
    token: AsyncFriendliToken
    image: AsyncImage
    endpoint: AsyncEndpoint

    def _init_sdks(self):
        self.chat = AsyncFriendliChat(self.sdk_configuration)
        self.completions = AsyncFriendliCompletions(self.sdk_configuration)
        self.token = AsyncFriendliToken(self.sdk_configuration)
        self.image = AsyncImage(self.sdk_configuration)
        self.endpoint = AsyncEndpoint(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from pydantic import model_serializer
from typing_extensions import NotRequired, TypedDict

from friendli.types import UNSET, UNSET_SENTINEL, BaseModel, Nullable, OptionalNullable


class ParametersTypedDict(TypedDict):
    r"""The parameters the functions accepts, described as a JSON Schema object.
    To represent a function with no parameters, use the value `{\"type\": \"object\", \"properties\": {}}`.

    """


class Parameters(BaseModel):
    r"""The parameters the functions accepts, described as a JSON Schema object.
    To represent a function with no parameters, use the value `{\"type\": \"object\", \"properties\": {}}`.

    """


class FunctionTypedDict(TypedDict):
    name: str
    r"""The name of the function to be called. Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64."""
    parameters: ParametersTypedDict
    r"""The parameters the functions accepts, described as a JSON Schema object.
    To represent a function with no parameters, use the value `{\"type\": \"object\", \"properties\": {}}`.

    """
    description: NotRequired[Nullable[str]]
    r"""A description of what the function does, used by the model to choose when and how to call the function."""


class Function(BaseModel):
    name: str
    r"""The name of the function to be called. Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length of 64."""

    parameters: Parameters
    r"""The parameters the functions accepts, described as a JSON Schema object.
    To represent a function with no parameters, use the value `{\"type\": \"object\", \"properties\": {}}`.

    """

    description: OptionalNullable[str] = UNSET
    r"""A description of what the function does, used by the model to choose when and how to call the function."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description"]
        nullable_fields = ["description"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const

from .functionresult import FunctionResult, FunctionResultTypedDict


class ToolCallResultTypedDict(TypedDict):
    function: FunctionResultTypedDict
    id: str
    r"""The ID of the tool call."""
    type: Literal["function"]
    r"""The type of the tool."""


class ToolCallResult(BaseModel):
    function: FunctionResult

    id: str
    r"""The ID of the tool call."""

    TYPE: Annotated[
        Annotated[Literal["function"], AfterValidator(validate_const("function"))],
        pydantic.Field(alias="type"),
    ] = "function"
    r"""The type of the tool."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing_extensions import TypedDict

from friendli.types import BaseModel


class ChatUsageTypedDict(TypedDict):
    completion_tokens: int
    r"""Number of tokens in the generated chat completions."""
    prompt_tokens: int
    r"""Number of tokens in the prompt."""
    total_tokens: int
    r"""Total number of tokens used in the request (`prompt_tokens` + `completion_tokens`)."""


class ChatUsage(BaseModel):
    completion_tokens: int
    r"""Number of tokens in the generated chat completions."""

    prompt_tokens: int
    r"""Number of tokens in the prompt."""

    total_tokens: int
    r"""Total number of tokens used in the request (`prompt_tokens` + `completion_tokens`)."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from abc import abstractmethod
from typing import Mapping, Optional

from friendli import models, utils
from friendli._hooks import HookContext
from friendli.types import UNSET, OptionalNullable
from friendli.utils import get_security_from_env

from .basesdk import AsyncSDK, BaseSDK, SyncSDK


class BaseImage(BaseSDK):
    pass


class SyncImage(BaseImage, SyncSDK):
    def generate(
        self,
        *,
        model: str,
        prompt: str,
        x_friendli_team: OptionalNullable[str] = UNSET,
        num_inference_steps: OptionalNullable[int] = 4,
        response_format: OptionalNullable[
            models.DedicatedImageGenerationBodyResponseFormat
        ] = "url",
        seed: OptionalNullable[int] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DedicatedImageGenerateSuccess:
        """Image generations

        Given a description, the model generates image(s).

        :param model: ID of target endpoint. If you want to send request to specific adapter, use the format \\"YOUR_ENDPOINT_ID:YOUR_ADAPTER_ROUTE\\". Otherwise, you can just use \\"YOUR_ENDPOINT_ID\\" alone.
        :param prompt: A text description of the desired image(s).
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param num_inference_steps: The number of inference steps to use during image generation. Supported range: [1, 10]. Defaults to 4.
        :param response_format: The format in which the generated image(s) will be returned. One of `url(default)`, `raw`, `png`, `jpeg`, and `jpg`.
        :param seed: The seed to use for image generation.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms
        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        request = models.DedicatedImagesGenerateRequest(
            x_friendli_team=x_friendli_team,
            dedicated_image_generation_body=models.DedicatedImageGenerationBody(
                model=model,
                prompt=prompt,
                num_inference_steps=num_inference_steps,
                response_format=response_format,
                seed=seed,
            ),
        )
        req = self._build_request(
            method="POST",
            path="/dedicated/v1/images/generations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_image_generation_body,
                False,
                False,
                "json",
                models.DedicatedImageGenerationBody,
            ),
            timeout_ms=timeout_ms,
        )
        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )
        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])
        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="dedicatedImagesGenerate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.DedicatedImageGenerateSuccess
            )
        if utils.match_response(http_res, ["422", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )


class AsyncImage(BaseImage, AsyncSDK):
    async def generate(
        self,
        *,
        model: str,
        prompt: str,
        x_friendli_team: OptionalNullable[str] = UNSET,
        num_inference_steps: OptionalNullable[int] = 4,
        response_format: OptionalNullable[
            models.DedicatedImageGenerationBodyResponseFormat
        ] = "url",
        seed: OptionalNullable[int] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.DedicatedImageGenerateSuccess:
        """Image generations

        Given a description, the model generates image(s).

        :param model: ID of target endpoint. If you want to send request to specific adapter, use the format \\"YOUR_ENDPOINT_ID:YOUR_ADAPTER_ROUTE\\". Otherwise, you can just use \\"YOUR_ENDPOINT_ID\\" alone.
        :param prompt: A text description of the desired image(s).
        :param x_friendli_team: ID of team to run requests as (optional parameter).
        :param num_inference_steps: The number of inference steps to use during image generation. Supported range: [1, 10]. Defaults to 4.
        :param response_format: The format in which the generated image(s) will be returned. One of `url(default)`, `raw`, `png`, `jpeg`, and `jpg`.
        :param seed: The seed to use for image generation.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms
        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        request = models.DedicatedImagesGenerateRequest(
            x_friendli_team=x_friendli_team,
            dedicated_image_generation_body=models.DedicatedImageGenerationBody(
                model=model,
                prompt=prompt,
                num_inference_steps=num_inference_steps,
                response_format=response_format,
                seed=seed,
            ),
        )
        req = self._build_request(
            method="POST",
            path="/dedicated/v1/images/generations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.dedicated_image_generation_body,
                False,
                False,
                "json",
                models.DedicatedImageGenerationBody,
            ),
            timeout_ms=timeout_ms,
        )
        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )
        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])
        http_res = await self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="dedicatedImagesGenerate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.DedicatedImageGenerateSuccess
            )
        if utils.match_response(http_res, ["422", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const

from .videodata import VideoData, VideoDataTypedDict


class VideoContentTypedDict(TypedDict):
    video_url: VideoDataTypedDict
    type: Literal["video_url"]
    r"""The type of the message content."""


class VideoContent(BaseModel):
    video_url: VideoData

    TYPE: Annotated[
        Annotated[Literal["video_url"], AfterValidator(validate_const("video_url"))],
        pydantic.Field(alias="type"),
    ] = "video_url"
    r"""The type of the message content."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations

from typing import Literal

import pydantic
from pydantic.functional_validators import AfterValidator
from typing_extensions import Annotated, TypedDict

from friendli.types import BaseModel
from friendli.utils import validate_const


class TextContentTypedDict(TypedDict):
    text: str
    r"""The text content of the message."""
    type: Literal["text"]
    r"""The type of the message content."""


class TextContent(BaseModel):
    text: str
    r"""The text content of the message."""

    TYPE: Annotated[
        Annotated[Literal["text"], AfterValidator(validate_const("text"))],
        pydantic.Field(alias="type"),
    ] = "text"
    r"""The type of the message content."""
